/**
 * \file InterfaceAquisicao.c
 * \brief Arquivo de implementao da interface com a circuitaria de aquisio.
 *
 * \author Universidade Tecnolgica Federal do Paran - UTFPR
 * \author Laboratrio de Automao e Sistemas de Controle Avanado - LASCA
 * \author ENGAUT Engenharia e Automao LTDA - ME
 *
 * Projeto CAMITATIBA - Arquivo de implementao da interface para interao com a circuitaria
 * de aquisio da Placa de Aquisio v2.0.
 *
 * Revises: nenhuma
 *
 * \date 01/04/2011
 * \author Rafael Jos Daciuk
 */

/*==================================================================
 * Incluses de Arquivos
 ==================================================================*/
#include "SonicLab.h"
#include "SonicLabDriver.h"
#include "Interfaces/InterfaceAquisicao.h"
#include "Interfaces/InterfaceI2C.h"
#include "Interfaces/InterfaceSi5338.h"
#include "Interfaces/InterfaceADS612x.h"

#include "RegistradoresSi5338.h"

#include <stdlib.h>

/*==================================================================
 * Variveis de uso Global no arquivo
 ==================================================================*/
Uint16 ganhosAmps = 0x0000;	///< Varivel Global que mantm o estado de configurao dos Amplificadores
Uint32 configFifosAds = 0x7888;	///< Varivel Global que mantm o estado de configurao das FIFOs e dos ADs

Uint16 * bufferCh1;	///< Buffer que armazenar os sinais de ultrassom adquiridos a partir do Canal 1 da Aquisio
Uint16 * bufferCh2;	///< Buffer que armazenar os sinais de ultrassom adquiridos a partir do Canal 2 da Aquisio
Uint16 * bufferCh3;	///< Buffer que armazenar os sinais de ultrassom adquiridos a partir do Canal 3 da Aquisio

Bool canalCked1, canalCked2, canalCked3;	///< Variveis que indicam quais canais da Aquisio devem ter seus sinais adquiridos

Uint32 qtdeAmostras;	///< Varivel global que armazena a configurao indicando quantas amostras devero ser adquiridas de cada Canal da Aquisio
Uint32 tamanhoAmostra;	///< Varivel global que armazena a configurao indicando o tamanho de cada amostra que ser adquirida de cada Canal da Aquisio

Uint32 fifoIndiceAqui = 0;	///< ndice do buffer circular para entrada de novos dados da Aquisio
Uint32 fifoIndiceEnvio = 0;	///< ndice do buffer circular para retirada de dados para transmisso
Uint32 fifoTamanho = 0;		///< Tamanho do buffer circular para armazenar os dados da Aquisio
Uint32 fifoNumEnviar = 0;	///< Indicador de quantas ondas presentes no buffer circular no foram transmitidas

Bool flagScopeOn;

/*==================================================================
 * Prottipos de Funes Privadas do arquivo
 ==================================================================*/


/*==================================================================
 * Prottipos de Rotinas de Tratamento de Interrupes
 ==================================================================*/
interrupt void _AquisicaoExcitInt();
interrupt void _AquisicaoEdmaInt();

/*==================================================================
 * Implementaes de Funes
 ==================================================================*/

/**
 * \fn void Aquis_Inicializa()
 * \brief Funo que inicializa a interface com a parte de aquisio.
 *
 * Esta funo inicializa a interface de interao com a parte de aquisio de dados de ultrassom.
 *
 */
void Aquis_Inicializa()
{
	unsigned int i;

	// inicializaes da circuitaria de aquisio
	*(volatile unsigned int *)CS_AQUIS_CONF = 0x7888; // escrita nas FIFOs desabilitada, FIFOs e ADs ligados e sem reset
	*(volatile unsigned int *)CS_AMP_CONF = 0x0000; // ganho dos amplificadores igual a zero

	// configura o ganho dos amplificadores 0x0GGG
	Aquis_DefineGanhoAmp(AMP_CANAL1, GANHO_AMP_1);
	Aquis_DefineGanhoAmp(AMP_CANAL2, GANHO_AMP_2);
	Aquis_DefineGanhoAmp(AMP_CANAL3, GANHO_AMP_3);

	// cdigo para configurao inicial do gerador de clock dos ADs
	I2C_Inicializa();
	Si5338_Configura((RegData *)Reg_Store);

	/*
	 * inicializao da aquisio: reset das FIFOs e dos ADs
	 */
	*(unsigned volatile short *)CS_AQUIS_CONF = 0x0AAA;
	for (i = 0; i < 50; i++);
	*(unsigned volatile short *)CS_AQUIS_CONF = 0x7888;
	configFifosAds = 0x7888;
	for (i = 0; i < SI5338_DELAY; i++);

	/*
	 * inicializao dos conversores ADS612x
	 */
	// habilita serial dos ADs
	configFifosAds |= 0x0111;
	*(unsigned volatile int *)CS_AQUIS_CONF = configFifosAds;
	ADS612x_Inicializa();
	// desabilita serial dos ADs
	configFifosAds &= ~0x0111;
	*(unsigned volatile int *)CS_AQUIS_CONF = configFifosAds;

	canalCked1 = FALSE;
	canalCked2 = FALSE;
	canalCked3 = FALSE;

	flagScopeOn = TRUE;

	bufferCh1 = NULL;
	bufferCh2 = NULL;
	bufferCh3 = NULL;
}

/**
 * \fn void Aquis_DefineGanhoAmp()
 * \brief Funo que define o ganho de um amplificador.
 *
 * Esta funo altera o ganho de um dos amplificadores AD8369 na entrada dos canais de aquisio
 * de sinais de ultrassom.
 *
 * \param Uint8 amp amplificador que ter o ganho alterado.
 * \param Uint8 ganho valor do ganho que ser definido.
 *
 */
void Aquis_DefineGanhoAmp(Uint8 amp, Uint8 ganho)
{
	ganhosAmps &= ~(0xF << (amp*4));
	ganhosAmps |= ganho << (amp*4);
	*(unsigned volatile short *)CS_AMP_CONF = ganhosAmps;
}

/**
 * \fn void Aquis_CpldReset()
 * \brief Funo que envia um Reset para a CPLD da Placa de Aquisio v2.0.
 *
 * Esta funo fora um Reset na CPLD da Placa de Aquisio v2.0 ao enviar um pulso ao pino MRS# da CPLD
 * atravs do GPIO que foi configurado para esta funo (GP0_7).
 *
 */
void Aquis_CpldReset()
{
	Uint32 i;

	// define o valor da sada GP0_7 como 1, ativando o Reset
	//CSL_FINS(gpioRegs->BANK[GP0].SET_DATA, GPIO_SET_DATA_SET7, CSL_GPIO_SET_DATA_SET_SET);
	//gpioRegs->BANK[GPIO_CPLD_MRS_BANK].SET_DATA |= ((1 << GPIO_CPLD_MRS_PIN) << (16*(GPIO_CPLD_MRS_BANK % 2)));
	GPIO_SetaPino(GPIO_CPLD_MRS_BANK, GPIO_CPLD_MRS_PIN, 0);

	// aguarda um perodo para que o Reset seja detectado
	for(i = 0; i < 100000; i++);

	// define o valor da sada GP0_7 como 0, desativando o Reset
	//CSL_FINS(gpioRegs->BANK[GP0].CLR_DATA, GPIO_CLR_DATA_CLR7, CSL_GPIO_CLR_DATA_CLR_CLR);
	//gpioRegs->BANK[GPIO_CPLD_MRS_BANK].CLR_DATA |= ((1 << GPIO_CPLD_MRS_PIN) << (16*(GPIO_CPLD_MRS_BANK % 2)));
	GPIO_SetaPino(GPIO_CPLD_MRS_BANK, GPIO_CPLD_MRS_PIN, 1);
}

/**
 * \fn void Aquis_AlocaBuffersSinais()
 * \brief Funo que aloca a memria para os buffers de sinais de ultrassom.
 *
 * Esta funo realiza a alocao de memria para cada um dos 3 buffers que armazenam os sinais de ultrassom
 * adquiridos.
 * A alocao  realizada de acordo com a quantidade de amostras a ser armazenada bem como com o tamanho
 * de cada amostra.
 *
 */
void Aquis_AlocaBuffersSinais()
{
	// libera a memria atualmente alocada para os buffers
	free(bufferCh1);
	free(bufferCh2);
	free(bufferCh3);

	if(canalCked1 == 0x01)
	{
		bufferCh1 = malloc(qtdeAmostras*tamanhoAmostra*sizeof(*bufferCh1));

		if(bufferCh1 == NULL)
		{
			flagScopeOn = TRUE;
		}
	}
	else
	{
		bufferCh1 = NULL;
	}

	if(canalCked2 == 0x01)
	{
		bufferCh2 = malloc(qtdeAmostras*tamanhoAmostra*sizeof(*bufferCh2));

		if(bufferCh2 == NULL)
		{
			flagScopeOn = TRUE;
		}
	}
	else
	{
		bufferCh2 = NULL;
	}

	if(canalCked3 == 0x01)
	{
		bufferCh3 = malloc(qtdeAmostras*tamanhoAmostra*sizeof(*bufferCh3));

		if(bufferCh3 == NULL)
		{
			flagScopeOn = TRUE;
		}
	}
	else
	{
		bufferCh3 = NULL;
	}
}

/**
 * \fn void Aquis_HabilitaCanais()
 * \brief Funo que define quais canais devem ter seus sinais adquiridos.
 *
 * Esta funo seleciona os canais que devero ter seus sinais adquiridos a partir das FIFO's para a memria
 * externa DDR2.
 *
 * \param Bool habilitaCanal1 se TRUE, adquire o Canal 1, se FALSE, no adquire.
 * \param Bool habilitaCanal2 se TRUE, adquire o Canal 2, se FALSE, no adquire.
 * \param Bool habilitaCanal3 se TRUE, adquire o Canal 3, se FALSE, no adquire.
 *
 */
void Aquis_HabilitaCanais(Bool habilitaCanal1, Bool habilitaCanal2, Bool habilitaCanal3)
{
	canalCked1 = habilitaCanal1;
	canalCked2 = habilitaCanal2;
	canalCked3 = habilitaCanal3;
}

/**
 * \fn void Aquis_Adquirir()
 * \brief Funo que implementa a rotina principal de aquisio de sinais a partir dos conversores A/D.
 *
 * Esta funo implementa a rotina principal de aquisio dos sinais de ultrassom, configurando a velocidade
 * de acesso  memria para um valor mais alto, verificando se as FIFO's contendo os sinais j esto
 * cheias e desabilitando a escrita nestas, voltando a velocidade de acesso  memria ao valor mais baixo,
 * sinalizando  rotina de processamento principal que a aquisio foi realizada e solicitando  EDMA a
 * transferncia dos dados.
 *
 */
void Aquis_Adquirir()
{
	// aumenta a velocidade de acesso  memria assncrona.
	EMIFA_Cs2AumentaVeloc();

	/*
	 * Verifica se a(s) FIFO('s) do(s) Canal(ais) de que se deve adquirir os dados
	 * est(o) cheia(s).
	 */
	if(canalCked1 == 0x01)
	{
		// verifica se a FIFO do Canal 1 est cheia
		while(((*(unsigned volatile int *)CS_AD_FLAGS & 0x00000002)>>1));
	}
	if(canalCked2 == 0x01)
	{
		// verifica se a FIFO do Canal 2 est cheia
		while(((*(unsigned volatile int *)CS_AD_FLAGS & 0x00000020)>>1));
	}
	if(canalCked3 == 0x01)
	{
		// verifica se a FIFO do Canal 3 est cheia
		while(((*(unsigned volatile int *)CS_AD_FLAGS & 0x00000200)>>1));
	}

	/*
	 * Desabilita a escrita nas FIFO's a partir dos AD's.
	 */
	configFifosAds |= 0x0888;
	*(unsigned volatile int *)CS_AQUIS_CONF = configFifosAds;

	// diminui a velocidade de acesso  memria assncrona.
	EMIFA_Cs2DiminuiVeloc();

	// sinaliza  rotina de processamento principal que a aquisio foi realizada.
	flagScopeOn = TRUE;

	// dispara a transferncia dos dados adquiridos para os buffers em memria externa via EDMA.
	Aquis_SalvarOndas();
}

/**
 * \fn void Aquis_SalvarOndas()
 * \brief Funo que solicita  EDMA a transferncia das ondas de ultrassom.
 *
 * Esta funo faz a requisio  EDMA da transferncia dos sinais de ultrassom presentes nas FIFO's dos
 * AD's para os buffers de armazenamento na memria externa DDR2.
 * A funo atualiza a posio dos destinos para onde as informaes sero copiadas e ento fora o disparo
 * da transferncia do canal EDMA configurado para o servio.
 * A funo tambm atualiza as informaes relativas ao buffer circular de armazenamento de dados.
 *
 */
void Aquis_SalvarOndas()
{
	Uint16 *destinoCh1, *destinoCh2, *destinoCh3;

	/*
	 * define a posio nos buffers onde devero ser gravadas as novas amostras.
	 */
	if (bufferCh1 != NULL)
	{
		destinoCh1 = (Uint16 *)((Uint32)bufferCh1) + (sizeof(*bufferCh1)*fifoIndiceAqui*tamanhoAmostra);
	}
	else
	{
		destinoCh1 = NULL;
	}

	if (bufferCh2 != NULL)
	{
		destinoCh2 = (Uint16 *)((Uint32)bufferCh2) + (sizeof(*bufferCh2)*fifoIndiceAqui*tamanhoAmostra);
	}
	else
	{
		destinoCh2 = NULL;
	}

	if (bufferCh3 != NULL)
	{
		destinoCh3 = (Uint16 *)((Uint32)bufferCh3) + (sizeof(*bufferCh3)*fifoIndiceAqui*tamanhoAmostra);
	}
	else
	{
		destinoCh3 = NULL;
	}

	// atualiza os parmetros da transferncia por EDMA.
	EDMA3_ConfFifoAd(tamanhoAmostra, destinoCh1, destinoCh2, destinoCh3);

	// dispara o incio da transferncia dos dados via EDMA.
	EDMA3_DisparaFifoAd();

	/*
	 * atualiza indices do buffer circular de armazenamento de dados.
	 */
	fifoIndiceAqui++;
	// verifica se chegou ao fim do buffer.
	if(fifoIndiceAqui >= fifoTamanho)
	{
		// se sim, volta o indice para o incio da memoria.
		fifoIndiceAqui = 0;
	}

	// verifica se o ndice de aquisio ultrapassou o de envio (aquisio est mais rpido que a transmisso).
	if(fifoIndiceAqui == fifoIndiceEnvio)
	{
		// se sim, incrementa o envio e perde-se uma onda, para evitar perder tudo o que ainda no foi enviado.
		fifoIndiceEnvio++;
		// verifica se o ndice de envio chegou ao fim do buffer.
		if(fifoIndiceEnvio >= fifoTamanho)
		{
			// volta o indice para o incio do buffer.
			fifoIndiceEnvio = 0;

		}
		// decrementa a quantidade de ondas que ainda no foram transmitidas (perdeu-se uma onda).
		fifoNumEnviar--;
	}

	// incrementa a quantidade de ondas que ainda no foram transmitidas (foi adquirida uma nova onda).
	fifoNumEnviar++;

	// TODO Se  aquisicao de massa verifica se adquiriu todas as ondas
//	if(!flagMassaOn)
//	{
//		tamanhoOndas--;
//		if(tamanhoOndas<=0)
//		{
//			DADOENVCTRLREG = 0x1000;
//
//			ExcitConfig();
//		}
//	}
}

/**
 * \fn void Aquis_GravaQtdeAmostras()
 * \brief Funo que define quantas amostras devero ser adquiridas.
 *
 * Esta funo define o parmetro de configurao que indica quantas amostras devero ser
 * adquiridas de cada Canal da Aquisio.
 *
 * \param Uint32 qtde quantidade de amostras a serem adquiridas.
 *
 */
void Aquis_GravaQtdeAmostras(Uint32 qtde)
{
	qtdeAmostras = qtde;
}

/**
 * \fn void Aquis_GravaQtdeAmostras()
 * \brief Funo que define o tamanho de cada amostra adquirida.
 *
 * Esta funo define o parmetro de configurao que indica o tamanho de cada amostra que
 * ser adquirida de cada Canal da Aquisio.
 *
 * \param Uint32 tamanho quantidade de amostras a serem adquiridas.
 *
 */
void Aquis_GravaTamanhoAmostra(Uint32 tamanho)
{
	tamanhoAmostra = tamanho;
}

/**
 * \fn void Aquis_ConfiguraBuffers()
 * \brief Funo que define configura os buffers de aquisio de sinais.
 *
 * Esta funo realiza a configurao dos buffers que iro armazenar os dados de ultrassom adquiridos.
 * Nesta funo  realizada a alocao da memria para os buffers de acordo com as configuraes de
 * quantidade de amostras por canal e de tamanho das amostras, bem como de quais canais sero adquiridos.
 * A funo tambm solicita a reconfigurao dos parmetros de transferncia da EDMA responsvel pela
 * cpia dos dados adquiridos para os buffers.
 *
 */
void Aquis_ConfiguraBuffers()
{
	Aquis_AlocaBuffersSinais();

	EDMA3_ConfFifoAd(tamanhoAmostra, bufferCh1, bufferCh2, bufferCh3);
}

/*==================================================================
 * Implementaes de Rotinas de Tratamento de Interrupes
 ==================================================================*/

/**
 * \fn interrupt void _AquisicaoExcitInt()
 * \brief Funo de tratamento de interrupo da Aquisio devido  Excitao.
 *
 * Esta funo trata o evento de interrupo disparado pela Controladora de Excitao quando os transdutores
 * so excitados.
 * A funo sinaliza para a rotina de processamento principal que os transdutores foram excitados para que
 * esta ento possa realizar a aquisio dos sinais de ultrassom.
 *
 */
interrupt void _AquisicaoExcitInt()
{
	// limpa o flag do evento gerado
	DSPINTC_LimpaEvento(GPIO_EXCIT_INT_VECT, GPIO_EXCIT_INT_EVT);

	// sinaliza para a rotina de processamento principal que os transdutores foram excitados
}

/**
 * \fn interrupt void _AquisicaoEdmaInt()
 * \brief Funo de tratamento de interrupo da Aquisio devido  EDMA.
 *
 * Esta funo trata o evento de interrupo disparado pela EDMA.
 * A funo sinaliza para a rotina de processamento principal que a transferncia de dados via EDMA das FIFO's
 * dos AD's para a memria externa foi concluda.
 *
 */
interrupt void _AquisicaoEdmaInt()
{
	int i;

	// limpa o flag do evento gerado.
	DSPINTC_LimpaEvento(QDMA_FIFO_AD_VECT, QDMA_FIFO_AD_EVT);

	// loop que garante que todas as interrupes da EDMA sero tratadas, pois seno no sero geradas novas interrupes.
	while (EDMA3_LeInt() != 0)
	{
		// verifica se a interrupo foi gerada pelo canal usado nas transferncias das FIFO's dos AD's para a memria externa DDR2.
		if (EDMA3_LeInt() == (0x1 << QDMA_FIFO_AD_TCC))
		{
			// limpa o flag de interrupo de transferncia completada do canal.
			EDMA3_LimpaInt(QDMA_FIFO_AD_TCC);

			/*
			 * como a transferncia do nmero de amostras relevante foi completada, d um Reset nas FIFO's, descartando o restante.
			 */
			// abaixa o pino de Master Reset nas FIFO's.
			configFifosAds &= ~0x7000;
			*(unsigned volatile int *)CS_AQUIS_CONF = configFifosAds;
			// levanta o pino de Master Reset nas FIFO's.
			configFifosAds |= 0x7000;
			*(unsigned volatile int *)CS_AQUIS_CONF = configFifosAds;

			// reabilita Escrita nas FIFO's dos canais que tiverem sido selecionados para aquisio.
			if(canalCked1 == 0x01)
			{
				configFifosAds &= ~0x0008;
			}
			if(canalCked2 == 0x01)
			{
				configFifosAds &= ~0x0080;
			}
			if(canalCked3 == 0x01)
			{
				configFifosAds &= ~0x0800;
			}
			*(unsigned volatile int *)CS_AQUIS_CONF = configFifosAds;

			// sinaliza para a rotina de processamento principal que a transferncia de dados via EDMA foi concluda.
		}

		// verifica se alguma outra interrupo de EDMA imprevista foi gerada e limpa-a, pois seno no sero geradas novas interrupes.
		if ((EDMA3_LeInt() != 0)
				&& (EDMA3_LeInt() != (0x1 << QDMA_FIFO_AD_TCC))
				)
		{
			// varre todos os flags de interrupo e limpa o flag caso este no represente uma interrupo prevista no firmware.
			for (i = 0; i < 32; i++)
			{
				// limpa o flag de interrupo caso este no represente uma interrupo prevista no firmware.
				if (i != QDMA_FIFO_AD_TCC)
				{
					EDMA3_LimpaInt(i);
				}
			}
		}
	}
}
