/**
 * \file InterfaceExcitacao.c
 * \brief Arquivo de implementao da interface com a Placa de Excitacao v1.0.
 *
 * \author Universidade Tecnolgica Federal do Paran - UTFPR
 * \author Laboratrio de Automao e Sistemas de Controle Avanado - LASCA
 * \author ENGAUT Engenharia e Automao LTDA - ME
 *
 * Projeto CAMITATIBA - Arquivo de implementao da interface para interao com a Placa de Excitacao v1.0.
 *
 * Revises: nenhuma
 *
 * \date 02/04/2011
 * \author Rafael Jos Daciuk
 */

/*==================================================================
 * Incluses de Arquivos
 ==================================================================*/
#include "SonicLab.h"
#include "SonicLabDriver.h"
#include "Interfaces/InterfaceExcitacao.h"

/*==================================================================
 * Variveis de uso Global no arquivo
 ==================================================================*/
ExcitConfigEstrutura excitConf;	///< Varivel global da estrutura de registradores da controladora de excitao

/*==================================================================
 * Prottipos de Funes Privadas do arquivo
 ==================================================================*/
Uint16 ExcitacaoMontaReg(Uint8 enderecoReg);
Uint16 *ExcitacaoRetornaReg(Uint8 enderecoReg);

/*==================================================================
 * Implementaes de Funes
 ==================================================================*/

/**
 * \fn void ExcitacaoInicializa()
 * \brief Funo que inicializa a interface com a controladora de excitao.
 *
 * Esta funo inicializa a interface com a controladora de excitao da Placa
 * de Excitao v1.0.
 *
 */
void ExcitacaoInicializa()
{
	/*
	 * Inicializao a estrutura global de registradores com valores default
	 */
	excitConf.regCtrlReg = CTRLREG_EXCIT_DESL;
	excitConf.regCh1Pw = CH1PW_INICIAL;
	excitConf.regCh2Pw = CH2PW_INICIAL;
	excitConf.regCh3Pw = CH3PW_INICIAL;
	excitConf.regExcitCnt = EXCITCNT_INICIAL;
	excitConf.regExcitPrdH = EXCITPRDH_INICIAL;
	excitConf.regExcitPrdL = EXCITPRDL_INICIAL;

	// envia configurao inicial para a controladora de excitao
	ExcitacaoConfigura();
}

/**
 * \fn void ExcitacaoConfigura()
 * \brief Funo que envia as configuraes para a controladora de excitao.
 *
 * Esta funo envia as configuraes presentes na estrutura de registradores global para a controladora
 * na Placa de Excitao v1.0, configurando-a.
 *
 */
void ExcitacaoConfigura()
{
	Uint16 regEnvio;

	// primeiramente desliga a excitao
	ExcitacaoDesliga();

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o registrador CH1PW da controladora de excitao
	regEnvio = ExcitacaoMontaReg(ENDERECO_CH1PW);
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o registrador CH2PW da controladora de excitao
	regEnvio = ExcitacaoMontaReg(ENDERECO_CH2PW);
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o registrador CH3PW da controladora de excitao
	regEnvio = ExcitacaoMontaReg(ENDERECO_CH3PW);
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o registrador EXCITCNT da controladora de excitao
	regEnvio = ExcitacaoMontaReg(ENDERECO_EXCITCNT);
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o registrador EXCITPRDH da controladora de excitao
	regEnvio = ExcitacaoMontaReg(ENDERECO_EXCITPRDH);
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o registrador EXCITPRDL da controladora de excitao
	regEnvio = ExcitacaoMontaReg(ENDERECO_EXCITPRDL);
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o registrador CTRLREG da controladora de excitao
	regEnvio = ExcitacaoMontaReg(ENDERECO_CTRLREG);
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);
}

/**
 * \fn void ExcitacaoDesliga()
 * \brief Funo que envia um comando para parar a excitao.
 *
 * Esta funo escreve no registrador de controle da controladora de excitao o valor para
 * que a excitao seja parada, caso esteja em andamento.
 *
 */
void ExcitacaoDesliga()
{
	Uint16 regEnvio;

	// aguarda a SPI da excitao ficar disponvel para envio de dados
	while (!SPI_VerificaEnvio(SPI_EXCIT));
	// configura o valor a ser enviado via SPI para escrever no registrador CTRLREG o comando de desligamento
	regEnvio = (ENDERECO_CTRLREG << 13)
			| (0x1 << 12)
			| (CTRLREG_EXCIT_DESL & 0xFFF);
	// envia o dado pela SPI
	SPI_Envia(SPI_EXCIT, SPI_EXCIT_FORMAT, SPI_EXCIT_SCSN, regEnvio);
}

/**
 * \fn Uint16 ExcitacaoMontaReg(Uint8 enderecoReg)
 * \brief Funo que monta o valor de um registrador para envio pela SPI.
 *
 * Esta funo monta o dado que deve ser enviado pela SPI para que o registrador desejado seja
 * configurado, inserindo o endereo do registrador no dado e setando o bit de escrita para a
 * controladora de excitao.
 *
 * \param Uint8 enderecoReg endereo do registrador que se quer configurar.
 *
 * \return dado a ser enviado pela SPI para configurar o registrador corretamente.
 *
 */
Uint16 ExcitacaoMontaReg(Uint8 enderecoReg)
{
	Uint16 regEnvio, valorReg;

	valorReg = ExcitacaoLeReg(enderecoReg);

	regEnvio = (enderecoReg << 13) // coloca o endereo do registrador que ser configurado nos bits 15:13
			| (0x1 << 12)	// seta o bit 12 indicando uma escrita no registrador
			| (valorReg & 0xFFF);	// coloca o valor a ser escrito no registrador nos bits 11:0

	return regEnvio;
}

/**
 * \fn void ExcitacaoGravaReg(Uint8 enderecoReg, Uint16 valorReg)
 * \brief Funo que grava o valor de um registrador na estrutura de configurao.
 *
 * \param Uint8 enderecoReg endereo do registrador que se quer alterar o valor.
 * \param Uint16 valorReg valor a ser escrito no registrador.
 *
 */
void ExcitacaoGravaReg(Uint8 enderecoReg, Uint16 valorReg)
{
	Uint16 *ponteiroReg;

	ponteiroReg = ExcitacaoRetornaReg(enderecoReg);

	*ponteiroReg = valorReg;
}

/**
 * \fn Uint16 ExcitacaoGravaReg(Uint8 enderecoReg)
 * \brief Funo que devolve o valor de um registrador na estrutura de configurao.
 *
 * \param Uint8 enderecoReg endereo do registrador do qual se quer obter o valor.
 *
 * \return valor do registrador presente na estrutura de configurao.
 *
 */
Uint16 ExcitacaoLeReg(Uint8 enderecoReg)
{
	Uint16 *ponteiroReg;

	ponteiroReg = ExcitacaoRetornaReg(enderecoReg);

	return *ponteiroReg;
}

/**
 * \fn Uint16 *ExcitacaoRetornaReg(Uint8 enderecoReg)
 * \brief Funo que devolve um registrador na estrutura de configurao.
 *
 * \param Uint8 enderecoReg endereo do registrador que se quer obter.
 *
 * \return ponteiro para o registrador na estrutura de configurao.
 *
 */
Uint16 *ExcitacaoRetornaReg(Uint8 enderecoReg)
{
	Uint16 *ponteiroReg;

	// verifica qual o registrador desejado
	switch (enderecoReg)
	{
	case ENDERECO_CTRLREG:
		ponteiroReg = &excitConf.regCtrlReg;
		break;
	case ENDERECO_CH1PW:
			ponteiroReg = &excitConf.regCh1Pw;
			break;
	case ENDERECO_CH2PW:
			ponteiroReg = &excitConf.regCh2Pw;
			break;
	case ENDERECO_CH3PW:
			ponteiroReg = &excitConf.regCh3Pw;
			break;
	case ENDERECO_EXCITCNT:
			ponteiroReg = &excitConf.regExcitCnt;
			break;
	case ENDERECO_EXCITPRDH:
			ponteiroReg = &excitConf.regExcitPrdH;
			break;
	case ENDERECO_EXCITPRDL:
			ponteiroReg = &excitConf.regExcitPrdL;
			break;
	default:
		// se no encontrar o registrador (endereo invlido), retorna NULL
		ponteiroReg = NULL;
	}

	// devolve o registrador desejado
	return ponteiroReg;
}
