/**
 * \file InterfaceI2C.c
 * \brief Arquivo de implementao do driver da interface I2C.
 *
 * \author Universidade Tecnolgica Federal do Paran - UTFPR
 * \author Laboratrio de Automao e Sistemas de Controle Avanado - LASCA
 * \author ENGAUT Engenharia e Automao LTDA - ME
 *
 * Projeto CAMITATIBA - Arquivo contendo a implementao do driver da interface I2C implementada
 * na CPLD da Placa de Aquisio v2.0.
 *
 * Revises: nenhuma
 *
 * \date 31/03/2011
 * \author Rafael Jos Daciuk
 */

/*==================================================================
 * Incluses de Arquivos
 ==================================================================*/
#include "SonicLab.h"
#include "Interfaces/InterfaceI2C.h"

/*==================================================================
 * Implementaes de Funes
 ==================================================================*/

/**
 * \fn void I2C_Inicializa()
 * \brief Funo que inicializa o controlador I2C/SMBus da CPLD.
 *
 */
void I2C_Inicializa()
{
	// desabilita o controlador (reset)
	*(unsigned volatile int *)CLOCK_CTRL_MBCR = 0x00;
	// habilita o controlador
	*(unsigned volatile int *)CLOCK_CTRL_MBCR = 0x80;
	// define o endereco I2C do dispositivo
	*(unsigned volatile int *)CLOCK_CTRL_MADR = 0x08;
	// limpa os flags de status do controlador
	*(unsigned volatile int *)CLOCK_CTRL_MBSR = 0x00;
}

/**
 * \fn void I2C_EnviaByte()
 * \brief Funo que escreve um byte via controlador I2C/SMBus da CPLD.
 *
 * Esta funo realiza o envio de um byte de dados para um perifrico I2C ligado ao controlador.
 * Atualmente, esta funo trata apenas a comunicao com o gerador de clocks da Placa de Aquisio
 * v2.0.
 *
 * \param unsigned char addr endereo do registrador a ser gravado.
 * \param unsigned char dat dado a ser gravado no registrador.
 *
 */
void I2C_EnviaByte(unsigned char addr, unsigned char dat)
{
	unsigned char smbusMbsr = 0x00;

	// aguarda o barramento I2C ficar livre testando o bit MBB do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x20;
	} while (smbusMbsr);

	// coloca o controlador em modo transmisso e joga o endereo do device no registrador de dados
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0x90;
	*(unsigned volatile char *)CLOCK_CTRL_MBDR = (CLOCK_I2C_ADDR << 1) & 0xFE; // o ltimo bit transmitido  ZERO, indicando uma escrita
	// seta o controlador como master gerando uma condicao de START iniciando a transmissao
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0xB0;

	// aguarda o fim da transferencia testando o bit MCF do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (!smbusMbsr);

	// ao trmino da transferencia coloca o endereo do registrador que ser gravado para ser enviado em seguida
	*(unsigned volatile char *)CLOCK_CTRL_MBDR = addr;

	// aguarda o bit MCF do controlador ser zerado, indicando o incio de uma nova transferencia
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (smbusMbsr);
	// aguarda o fim da transferencia testando o bit MCF do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (!smbusMbsr);

	// ao trmino da transferencia coloca o valor que ser gravado no registrador para ser enviado em seguida
	*(unsigned volatile char *)CLOCK_CTRL_MBDR = dat;

	// aguarda o bit MCF do controlador ser zerado, indicando o incio de uma nova transferencia
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (smbusMbsr);
	// aguarda o fim da transferencia testando o bit MCF do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (!smbusMbsr);

	// tira o controlador do modo master, gerando uma condicao de STOP e finalizando a transmissao
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0x90;
}

/**
 * \fn void I2C_RecebeByte()
 * \brief Funo que realiza a leitura de um byte via controlador I2C/SMBus da CPLD.
 *
 * Esta funo realiza a leitura de um byte de dados de um perifrico I2C ligado ao controlador.
 * Atualmente, esta funo trata apenas a comunicao com o gerador de clocks da Placa de Aquisio
 * v2.0.
 *
 * \param unsigned char addr endereo do registrador a ser lido.
 *
 * \return o valor do byte presente no registrador solicitado.
 *
 */
unsigned char I2C_RecebeByte(unsigned char addr)
{
	unsigned char smbusMbsr = 0x00;
	unsigned char smbusMbdr = 0x00;

	// aguarda o barramento I2C ficar livre testando o bit MBB do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x20;
	} while (smbusMbsr);

	// coloca o controlador em modo transmisso e joga o endereo do device no registrador de dados
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0x90;
	*(unsigned volatile char *)CLOCK_CTRL_MBDR = (CLOCK_I2C_ADDR << 1) & 0xFE; // o ltimo bit transmitido  ZERO, indicando uma escrita
	// seta o controlador como master gerando uma condicao de START iniciando a transmissao
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0xB0;

	// aguarda o fim da transferencia testando o bit MCF do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (!smbusMbsr);

	// ao trmino da transferencia coloca o endereo do registrador que ser lido para ser enviado em seguida
	*(unsigned volatile char *)CLOCK_CTRL_MBDR = addr;

	// aguarda o bit MCF do controlador ser zerado, indicando o incio de uma nova transferencia
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (smbusMbsr);
	// aguarda o fim da transferencia testando o bit MCF do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (!smbusMbsr);

	// tira o controlador do modo master, gerando uma condicao de STOP e finalizando a transmissao
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0x90;

	// aguarda o barramento I2C ficar livre testando o bit MBB do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x20;
	} while (smbusMbsr);

	// coloca o controlador em modo recepcao e joga o endereo do device no registrador de dados
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0x80;
	*(unsigned volatile char *)CLOCK_CTRL_MBDR = (CLOCK_I2C_ADDR << 1) | 0x01; // o ltimo bit transmitido  UM, indicando uma leitura
	// seta o controlador como master gerando uma condicao de START iniciando a transmissao
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0xA0;

	// aguarda o bit MCF do controlador ser zerado, indicando o incio de uma nova transferencia
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (smbusMbsr);
	// aguarda o fim da transferencia testando o bit MCF do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (!smbusMbsr);

	// realiza a leitura preliminar do dado presente no registrador de dados recebidos
	smbusMbdr = *(unsigned volatile char *)CLOCK_CTRL_MBDR;

	// aguarda o bit MCF do controlador ser zerado, indicando o incio de uma nova transferencia
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (smbusMbsr);

	// nega o bit de ACKNOWLEDGE para indicar ao device que a transferencia acabou
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0xA8;

	// aguarda o fim da transferencia testando o bit MCF do controlador
	do {
		smbusMbsr = *(unsigned volatile char *)CLOCK_CTRL_MBSR;
		smbusMbsr &= 0x80;
	} while (!smbusMbsr);

	// nega o bit de ACKNOWLEDGE para indicar ao device que a transferencia acabou
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0xA8;

	// realiza a leitura do dado presente no registrador de dados recebidos
	smbusMbdr = *(unsigned volatile char *)CLOCK_CTRL_MBDR;

	// tira o controlador do modo master, gerando uma condicao de STOP e finalizando a transmissao
	*(unsigned volatile char *)CLOCK_CTRL_MBCR = 0x80;

	return smbusMbdr;
}
