/**
 * \file InterfaceSi5338.c
 * \brief Arquivo de implementao do driver do Si5338.
 *
 * \author Universidade Tecnolgica Federal do Paran - UTFPR
 * \author Laboratrio de Automao e Sistemas de Controle Avanado - LASCA
 * \author ENGAUT Engenharia e Automao LTDA - ME
 *
 * Projeto CAMITATIBA - Arquivo de implementao do driver do gerador de clock dos
 * A/D's da aquisio, o Si5338.
 *
 * Revises: nenhuma
 *
 * \date 31/03/2011
 * \author Rafael Jos Daciuk
 */

/*==================================================================
 * Incluses de Arquivos
 ==================================================================*/
#include "SonicLab.h"
#include "Interfaces/InterfaceSi5338.h"
#include "Interfaces/InterfaceI2C.h"

/*==================================================================
 * Implementaes de Funes
 ==================================================================*/

/**
 * \fn void Si5338_Configura()
 * \brief Funo que configura o gerador de clock Si5338.
 *
 * Esta funo configura o gerador de clock dos A/D's da aquisio, o Si5338.
 * Para realizar a configurao, um conjunto de parmetros presente no arquivo
 * register_map.h  enviado via I2C para o gerador de clock.
 * Estes parmetros foram gerados a partir do ClockBuilder Desktop software version 2.7,
 * fornecido pelo fabricante do chip.
 *
 * \param const Reg_Data *Reg_Store ponteiro para a estrutura contendo os valores dos registradores.
 *
 */
void Si5338_Configura(const RegData *Reg_Store)
{
	int counter;
	RegData curr;
	unsigned char curr_chip_val, clear_curr_val, clear_new_val, combined, reg;

	// delay added to wait for Si5338 to be ready to communicate
	// after turning on
	for (counter = 0; counter < SI5338_DELAY; counter++);

	I2C_EnviaByte(230, 0x10);		//OEB_ALL = 1
	I2C_EnviaByte(241, 0xE5);		//DIS_LOL = 1

	// for all the register values in the Reg_Store array
	// get each value and mask and apply it to the Si5338
	for(counter = 0; counter < NUM_REGS_MAX; counter++)
	{
		curr = Reg_Store[counter];
		if(curr.Reg_Mask != 0x00)
		{
			if(curr.Reg_Mask == 0xFF)
			{
				// do a write transaction only
				// since the mask is all ones
				I2C_EnviaByte(curr.Reg_Addr, curr.Reg_Val);
			}
			else
			{
				// do a read-modify-write
				curr_chip_val = I2C_RecebeByte(curr.Reg_Addr);
				clear_curr_val = curr_chip_val & ~curr.Reg_Mask;
				clear_new_val = curr.Reg_Val & curr.Reg_Mask;
				combined = clear_new_val | clear_curr_val;
				I2C_EnviaByte(curr.Reg_Addr, combined);
			}
		}
	}

	// check LOS alarm for the xtal input
	// on IN1 and IN2 (and IN3 if necessary) -
	// change this mask if using inputs on IN4, IN5, IN6
	reg = I2C_RecebeByte(218) & LOS_MASK;
	while(reg != 0)
	{
		reg = I2C_RecebeByte(218) & LOS_MASK;
	}

	I2C_EnviaByte(49, I2C_RecebeByte(49) & 0x7F);	// FCAL_OVRD_EN = 0
	I2C_EnviaByte(246, 2);						// soft reset
	I2C_EnviaByte(241, 0x65);					// DIS_LOL = 0

	// wait for Si5338 to be ready after calibration (ie, soft reset)
	for (counter = 0; counter < SI5338_DELAY; counter++);
	for (counter = 0; counter < SI5338_DELAY; counter++);

	// make sure the device locked by checking PLL_LOL and SYS_CAL
	reg = I2C_RecebeByte(218) & LOCK_MASK;
	while(reg != 0)
	{
		reg = I2C_RecebeByte(218) & LOCK_MASK;
	}

	// copy FCAL values
	I2C_EnviaByte(45, I2C_RecebeByte(235));
	I2C_EnviaByte(46, I2C_RecebeByte(236));

	// clear bits 0 and 1 from 47 and
	// combine with bits 0 and 1 from 237
	reg = (I2C_RecebeByte(47) & 0xFC) | (I2C_RecebeByte(237) & 3);
	I2C_EnviaByte(47, reg);
	I2C_EnviaByte(49, I2C_RecebeByte(49) | 0x80);	// FCAL_OVRD_EN = 1
	I2C_EnviaByte(230, 0x00);					// OEB_ALL = 0
}
