/**
 * \file SonicLabDriver.c
 * \brief Arquivo de implementao dos drivers dos perifricos do OMAP-L138.
 *
 * \author Universidade Tecnolgica Federal do Paran - UTFPR
 * \author Laboratrio de Automao e Sistemas de Controle Avanado - LASCA
 * \author ENGAUT Engenharia e Automao LTDA - ME
 *
 * Projeto CAMITATIBA - Arquivo de implementao dos drivers dos perifricos do OMAP-L138.
 *
 * Revises: nenhuma
 *
 * \date 24/03/2011
 * \author Rafael Jos Daciuk
 */

/*==================================================================
 * Incluses de Arquivos
 ==================================================================*/
#include "SonicLab.h"
#include "Interfaces/SonicLabDriver.h"

#include <c6x.h>

//CSLR
#include <ti/psp/cslr/soc_OMAPL138.h>
#include <ti/psp/cslr/csl_types.h>
#include <ti/psp/cslr/cslr.h>
#include <ti/psp/cslr/cslr_dspintc.h>
#include <ti/psp/cslr/cslr_emac1.h>
#include <ti/psp/cslr/cslr_tmr.h>
#include <ti/psp/cslr/cslr_psc_OMAPL138.h>
#include <ti/psp/cslr/cslr_pllc_OMAPL138.h>
#include <ti/psp/cslr/cslr_syscfg0_OMAPL138.h>
#include <ti/psp/cslr/cslr_syscfg1_OMAPL138.h>
#include <ti/psp/cslr/cslr_ddr2_mddr.h>
#include <ti/psp/cslr/cslr_emifa2.h>
#include <ti/psp/cslr/cslr_edma3cc.h>
#include <ti/psp/cslr/cslr_spi.h>
#include <ti/psp/cslr/cslr_gpio.h>

// typedefs que esto faltando na CSLR
typedef volatile CSL_EmacRegs         *CSL_EmacRegsOvly;
//Fim CSLR

/*==================================================================
 * Variveis Globais do Firmware
 ==================================================================*/
extern void _intcVectorTable();		///< Ponteiro para o incio do vetor de interrupes

/*==================================================================
 * Variveis de uso Global no arquivo
 ==================================================================*/
CSL_PscRegsOvly psc0Regs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;	///< Estrutura contendo os registradores do PSC0 (Power and Sleep Controller 0)
CSL_PscRegsOvly psc1Regs = (CSL_PscRegsOvly)CSL_PSC_1_REGS;	///< Estrutura contendo os registradores do PSC1 (Power and Sleep Controller 1)
CSL_Ddr2_MddrRegsOvly ddr2Regs = (CSL_Ddr2_MddrRegsOvly)CSL_DDR2_0_CTRL_REGS;	///< Estrutura contendo os registradores do DDR2 Controller
CSL_SyscfgRegsOvly syscfgRegs = (CSL_SyscfgRegsOvly)CSL_SYSCFG_0_REGS;	///< Estrutura contendo os registradores do SYSCFG (System Configuration Module)
CSL_Syscfg1RegsOvly syscfg1Regs = (CSL_Syscfg1RegsOvly)CSL_SYSCFG_1_REGS;	///< Estrutura contendo os registradores do SYSCFG1 (System Configuration Module 1)
CSL_PllcRegsOvly pllc0Regs = (CSL_PllcRegsOvly)CSL_PLLC_0_REGS;	///< Estrutura contendo os registradores do PLLC0 (Phase Locked Loop Controller 0)
CSL_PllcRegsOvly pllc1Regs = (CSL_PllcRegsOvly)CSL_PLLC_1_REGS;	///< Estrutura contendo os registradores do PLLC1 (Phase Locked Loop Controller 1)
CSL_EmifaRegsOvly emifaRegs = (CSL_EmifaRegsOvly)CSL_EMIFA_0_REGS;	///< Estrutura contendo os registradores do EMIFA (External Memory Interface A)
CSL_Edma3ccRegsOvly edma30ccRegs = (CSL_Edma3ccRegsOvly)CSL_EDMA30CC_0_REGS;	///< Estrutura contendo os registradores do EDMA30CC (Enhanced Direct Memory Access 0 Channel Controller)
CSL_SpiRegsOvly spi1Regs = (CSL_SpiRegsOvly)CSL_SPI_1_REGS;	///< Estrutura contendo os registradores da SPI1 (Serial Peripheral Interface 1)
CSL_TmrRegsOvly timer2Regs = (CSL_TmrRegsOvly)CSL_TMR_2_REGS;	///< Estrutura contendo os registradores do Timer 2
CSL_GpioRegsOvly gpioRegs = (CSL_GpioRegsOvly)CSL_GPIO_0_REGS;	///< Estrutura contendo os registradores do GPIO (General Purpose Input/Output Peripheral)
CSL_DspintcRegsOvly intcRegs = (CSL_DspintcRegsOvly)CSL_INTC_0_REGS;	///< Estrutura contendo os registradores do INTC (C674x Megamodule Interrupt Controller)

Uint32 emifCe2cfgSpeedDown;	///< Configurao do espao CS2 da EMIFA para acesso lento aos perifricos
Uint32 emifCe2cfgSpeedUp;	///< Configurao do espao CS2 da EMIFA para acesso rpido aos perifricos

CSL_Edma3ccParamSetRegs paramInicialCopiaFifos;	///< Conjunto de parmetros da transferncia inicial de dados das FIFO's dos AD's

/*==================================================================
 * Implementaes de Funes
 ==================================================================*/

/**
 * \fn void InicializaPlaca()
 * \brief Funo que inicializa os perifricos primrios da placa.
 *
 * Esta funo realiza a inicializao dos perifricos primrios do processador, como:
 *		- PSC (Power and Sleep Controller): Inicializa o PSC colocando os perifricos em estado ENABLE
 *		- PLLC (Phase-Locked Loop Controller): Inicializa a PLL para gerar os clocks necessrios do processador
 */
void InicializaOmap()
{
	PSC_Inicializa();
	PLLC_Inicializa();
}

/**
 * \fn void InicializaMemorias()
 * \brief Funo que inicializa as interfaces de memria do processador.
 *
 * Esta funo realiza a inicializao das interfaces de memria do processador, como:
 *		- DDR2 (DDR2 Controller): Inicializa o controlador DDR2 para acessar a memria externa DDR2.
 *		- EMIFA (External Memory Interface A): Inicializa a EMIFA para acessar os perifricos mapeados em memria.
 */
void InicializaMemorias()
{
	DDR2_Inicializa();
	EMIFA_Inicializa();
}

/**
 * \fn void PSC_Inicializa()
 * \brief Funo que inicializa o PSC (Power and Sleep Controller).
 *
 */
void PSC_Inicializa()
{
	//-----------------------------------------
	// PSC0, domain 0 - all modules, always on.
	//-----------------------------------------

	// configure the next state for psc0 modules.

	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_EMIFA, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SPI0, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_MMCSD0, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_AINTC, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_UART0, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SCR0_SS, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SCR1_SS, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc0Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SCR2_SS, CSL_PSC_MDCTL_NEXT_ENABLE);


	//-----------------------------------------
	// PSC1, domain 0 - all modules, always on.
	//-----------------------------------------

	// configure the next state for psc1 modules.

//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_USB0, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_USB1, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_GPIO, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_UHPI, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_EMAC, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_MCASP0, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SATA, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_VPIF, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SPI1, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_I2C1, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_UART1, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_UART2, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_MCBSP0, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_MCBSP1, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_LCDC, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_EHRPWM, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_MMCSD1, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_UPP, CSL_PSC_MDCTL_NEXT_ENABLE);
//	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_ECAP0_1_2, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SCRF0_SS, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SCRF1_SS, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SCRF2_SS, CSL_PSC_MDCTL_NEXT_ENABLE);
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_SHRAM, CSL_PSC_MDCTL_NEXT_ENABLE);
}

/**
 * \fn		void PSC_MudaEstadoPerif(CSL_PscRegsOvly pscRegs, uint32_t domain, uint8_t module, uint8_t next_state)
 * \brief	Funo que muda o estado de um perifrico no PSC (Power and Sleep Controller).
 *
 * \param	CSL_PscRegsOvly pscRegs estrutura dos resgistradores da PSC que contm o mdulo.
 * \param	uint32_t domain power domain do mdulo (CSL_PSC_PTCMD_GO0_MASK/PD0 ou CSL_PSC_PTCMD_GO1_MASK/PD1, vide manual do PSC).
 * \param	uint8_t module mdulo a ser alterado.
 * \param	uint8_t next_state prximo estado do mdulo.
 *
 */
void PSC_MudaEstadoPerif(CSL_PscRegsOvly pscRegs, Uint32 domain, Uint8 module, Uint8 next_state)
{
	// spin until existing transitions are done.
	while (pscRegs->PTSTAT & domain);
	//while (CHKBIT(psc->PTSTAT, in_domain)) {}

	// if we are already in the requested state...just return.
	if (CSL_FEXT(pscRegs->MDSTAT[module], PSC_MDSTAT_STATE) == next_state)
	//if (CHKBIT(psc->MDSTAT[in_module], MASK_STATE) == in_next_state)
	{
		return;
	}

	// setup the transition...
	CSL_FINS(pscRegs->MDCTL[module], PSC_MDCTL_NEXT, next_state);
	//CLRBIT(psc->MDCTL[in_module], NEXT);
	//SETBIT(psc->MDCTL[in_module], in_next_state);

	// kick off the transition.
	pscRegs->PTCMD = domain;
	//SETBIT(psc->PTCMD, in_domain);

	// spin until transition is done.
	while (pscRegs->PTSTAT & domain);
	//while (CHKBIT(psc->PTSTAT, in_domain)) {}
	while(CSL_FEXT(pscRegs->MDSTAT[module], PSC_MDSTAT_STATE) != next_state);
	//while (CHKBIT(psc->MDSTAT[in_module], MASK_STATE) != in_next_state) {}
}

/**
 * \fn void PLLC_Inicializa()
 * \brief Funo que inicializa o PLLC (Phase-Locked Loop Controller).
 *
 * Esta funo configura e inicializa a PLL do OMAP-L138 a fim de gerar os clock's necessrios para seu funcionamento.
 *
 * Os clocks gerados de acordo com esta funo so:
 *
 *  - PLL0_SYSCLK1 = 300 MHz
 *  - PLL0_SYSCLK2 = 150 MHz
 *  - PLL0_SYSCLK3 = 50 MHz
 *  - PLL0_SYSCLK4 = 75 MHz
 *  - PLL0_SYSCLK5 = 100 MHz
 *  - PLL0_SYSCLK6 = 300 MHz
 *  - PLL0_SYSCLK7 = 50 MHz
 *  - PLL0_AUXCLK = 24 MHz
 *  - PLL0_OBSCLK = 24 MHz
 *
 *  - PLL1_SYSCLK1 = 300 MHz
 *  - PLL1_SYSCLK2 = 150 MHz
 *  - PLL1_SYSCLK3 = 100 MHz
 *  - PLL1_OBSCLK = 24 MHz
 *
 */
void PLLC_Inicializa()
{
	Uint32 i;

	// desbloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x83E70B13;
	syscfgRegs->KICK1R = 0x95A4F1E0;

	/*
	 *  Configurao da PLL0
	 */

	// unlock pll regs.
	CSL_FINST(syscfgRegs->CFGCHIP0, SYSCFG_CFGCHIP0_PLL_MASTER_LOCK, FREE);

	// select clock mode (on-chip oscillator).
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_CLKMODE, OSCIN);

	// prepare to enable pll (PLLENSRC must be clear for PLLEN to have effect).
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_PLLENSRC, CLEAR);

	// disable external clock source.
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_EXTCLKSRC, DISABLE);

	// switch to bypass mode...wait 4 cycles to ensure it switches properly.
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_PLLEN, BYPASS);
	for (i = 0; i < 4; i++);

	// reset the pll.
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_PLLRST, ASSERT);

	// PLL initialization sequence
	//----------------------------
	// power up the pll... clear power down bit.
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_PLLPWRDN, POWERED_UP);

	// program the required multiplier value.
	pllc0Regs->PLLM = 24; // multiplicador setado em 25, pll clock = 600 MHz

	// program postdiv ratio.
	CSL_FINS(pllc0Regs->POSTDIV, PLLC_POSTDIV_RATIO, 1); // postdiv setado em 2, clock base = 300 MHz
	CSL_FINST(pllc0Regs->POSTDIV, PLLC_POSTDIV_POSTDEN, ENABLE);

	// spin until all transitions are complete.
	while (CSL_FEXT(pllc0Regs->PLLSTAT, PLLC_PLLSTAT_GOSTAT));

	// program the divisors.
	CSL_FINS(pllc0Regs->PLLDIV1, PLLC_PLLDIV1_RATIO, 0); // plldiv1 setado em 1, pll0_sysclock1 = 300 MHz
	CSL_FINST(pllc0Regs->PLLDIV1, PLLC_PLLDIV1_D1EN, ENABLE);
	CSL_FINS(pllc0Regs->PLLDIV2, PLLC_PLLDIV2_RATIO, 1); // plldiv2 setado em 2, pll0_sysclock2 = 150 MHz
	CSL_FINST(pllc0Regs->PLLDIV2, PLLC_PLLDIV2_D2EN, ENABLE);
	CSL_FINS(pllc0Regs->PLLDIV3, PLLC_PLLDIV3_RATIO, 5); // plldiv3 setado em 6, pll0_sysclock3 = 50 MHz
	CSL_FINST(pllc0Regs->PLLDIV3, PLLC_PLLDIV3_D3EN, ENABLE);
	CSL_FINS(pllc0Regs->PLLDIV4, PLLC_PLLDIV4_RATIO, 3); // plldiv4 setado em 4, pll0_sysclock4 = 75 MHz
	CSL_FINST(pllc0Regs->PLLDIV4, PLLC_PLLDIV4_D4EN, ENABLE);
	CSL_FINS(pllc0Regs->PLLDIV5, PLLC_PLLDIV5_RATIO, 2); // plldiv5 setado em 3, pll0_sysclock5 = 100 MHz
	CSL_FINST(pllc0Regs->PLLDIV5, PLLC_PLLDIV5_D5EN, ENABLE);
	CSL_FINS(pllc0Regs->PLLDIV6, PLLC_PLLDIV6_RATIO, 0); // plldiv6 setado em 1, pll0_sysclock6 = 300 MHz
	CSL_FINST(pllc0Regs->PLLDIV6, PLLC_PLLDIV6_D6EN, ENABLE);
	CSL_FINS(pllc0Regs->PLLDIV7, PLLC_PLLDIV7_RATIO, 5); // plldiv7 setado em 6, pll0_sysclock7 = 50 MHz
	CSL_FINST(pllc0Regs->PLLDIV7, PLLC_PLLDIV7_D7EN, ENABLE);

	// kick off the transitions and spin until they are complete.
	CSL_FINST(pllc0Regs->PLLCMD, PLLC_PLLCMD_GOSET, PHASE_ALIGN);
	while (CSL_FEXT(pllc0Regs->PLLSTAT, PLLC_PLLSTAT_GOSTAT));

	/*Wait for PLL to reset properly. See PLL spec for PLL reset time - This step is not required here -step11*/
	for(i = 0; i < 200; i++);   /*128 MXI Cycles*/

	// bring pll out of reset and wait for pll to lock.
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_PLLRST, DEASSERT);
	for (i = 0; i < 2400; i++);

	// exit bypass mode.
	CSL_FINST(pllc0Regs->PLLCTL, PLLC_PLLCTL_PLLEN, PLL);

	// lock pll regs.
	CSL_FINST(syscfgRegs->CFGCHIP0, SYSCFG_CFGCHIP0_PLL_MASTER_LOCK, LOCK);

	/*
	 *  Configurao da PLL1
	 */

	// unlock pll regs.
	CSL_FINST(syscfgRegs->CFGCHIP3, SYSCFG_CFGCHIP3_PLL1_MASTER_LOCK, FREE);

	// prepare to enable pll (PLLENSRC must be clear for PLLEN to have effect).
	CSL_FINST(pllc1Regs->PLLCTL, PLLC_PLLCTL_PLLENSRC, CLEAR);

	// switch to bypass mode...wait 4 cycles to ensure it switches properly.
	CSL_FINST(pllc1Regs->PLLCTL, PLLC_PLLCTL_PLLEN, BYPASS);
	for (i = 0; i < 4; i++);

	// reset the pll.
	CSL_FINST(pllc1Regs->PLLCTL, PLLC_PLLCTL_PLLRST, ASSERT);

	// PLL initialization sequence
	//----------------------------
	// power up the pll... clear power down bit.
	CSL_FINST(pllc1Regs->PLLCTL, PLLC_PLLCTL_PLLPWRDN, POWERED_UP);

	// program the required multiplier value.
	pllc1Regs->PLLM = 24; // multiplicador setado em 25, pll clock = 600 MHz

	// program postdiv ratio.
	CSL_FINS(pllc1Regs->POSTDIV, PLLC_POSTDIV_RATIO, 1); // postdiv setado em 2, clock base = 300 MHz
	CSL_FINST(pllc1Regs->POSTDIV, PLLC_POSTDIV_POSTDEN, ENABLE);

	// spin until all transitions are complete.
	while (CSL_FEXT(pllc1Regs->PLLSTAT, PLLC_PLLSTAT_GOSTAT));

	// program the divisors.
	CSL_FINS(pllc1Regs->PLLDIV1, PLLC_PLLDIV1_RATIO, 0); // plldiv1 setado em 1, pll1_sysclock1 = 300 MHz
	CSL_FINST(pllc1Regs->PLLDIV1, PLLC_PLLDIV1_D1EN, ENABLE);
	CSL_FINS(pllc1Regs->PLLDIV2, PLLC_PLLDIV2_RATIO, 1); // plldiv2 setado em 2, pll1_sysclock2 = 150 MHz
	CSL_FINST(pllc1Regs->PLLDIV2, PLLC_PLLDIV2_D2EN, ENABLE);
	CSL_FINS(pllc1Regs->PLLDIV3, PLLC_PLLDIV3_RATIO, 2); // plldiv3 setado em 3, pll1_sysclock3 = 100 MHz
	CSL_FINST(pllc1Regs->PLLDIV3, PLLC_PLLDIV3_D3EN, ENABLE);

	// kick off the transitions and spin until they are complete.
	CSL_FINST(pllc1Regs->PLLCMD, PLLC_PLLCMD_GOSET, PHASE_ALIGN);
	while (CSL_FEXT(pllc1Regs->PLLSTAT, PLLC_PLLSTAT_GOSTAT));

	// bring pll out of reset and wait for pll to lock.
	CSL_FINST(pllc1Regs->PLLCTL, PLLC_PLLCTL_PLLRST, DEASSERT);
	for (i = 0; i < 2400; i++);

	// exit bypass mode.
	CSL_FINST(pllc1Regs->PLLCTL, PLLC_PLLCTL_PLLEN, PLL);

	// lock pll regs.
	CSL_FINST(syscfgRegs->CFGCHIP3, SYSCFG_CFGCHIP3_PLL1_MASTER_LOCK, LOCK);

	// bloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x00000000;
	syscfgRegs->KICK1R = 0x00000000;
}

/**
 * \fn void DDR2_Inicializa()
 * \brief Funo que inicializa o controlador DDR2.
 *
 */
void DDR2_Inicializa()
{
	// desbloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x83E70B13;
	syscfgRegs->KICK1R = 0x95A4F1E0;

	// enable ddr2 clock.
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_DDR2_MDDR, CSL_PSC_MDCTL_NEXT_ENABLE);

	// check if vtp calibration is enabled.
	if (CSL_FEXT(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_POWERDN) & CSL_SYSCFG1_VTPIO_CTL_POWERDN_ENABLE)
	{
		// vtp cal disabled, begin cal.

		// enable input buffer and vtp.
		CSL_FINST(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_IOPWRDN, ENABLE);
		CSL_FINST(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_POWERDN, DISABLE);

		// pulse clrz to init vtp cal.
		CSL_FINS(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_CLKRZ, 1);
		CSL_FINS(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_CLKRZ, 0);
		CSL_FINS(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_CLKRZ, 1);

		// poll ready bit to wait for cal to complete.
		while(CSL_FEXT(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_READY) == CSL_SYSCFG1_VTPIO_CTL_READY_NOT_READY);

		// set lock and power save bits.
		CSL_FINST(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_PWRSAVE, ENABLE);
		CSL_FINST(syscfg1Regs->VTPIO_CTL, SYSCFG1_VTPIO_CTL_LOCK, LOCK);
	}

	// config ddr timing.
	CSL_FINST(ddr2Regs->DRPYC1R, DDR2_MDDR_DRPYC1R_EXT_STRBEN, EXT_STRB);
	CSL_FINST(ddr2Regs->DRPYC1R, DDR2_MDDR_DRPYC1R_PWRDNEN, PWREN);
	CSL_FINS(ddr2Regs->DRPYC1R, DDR2_MDDR_DRPYC1R_RL, 0x4);

	// config ddr slew rate.
	CSL_FINST(syscfg1Regs->DDR_SLEW, SYSCFG1_DDR_SLEW_CMOSEN, LVCMOS);
	CSL_FINST(syscfg1Regs->DDR_SLEW, SYSCFG1_DDR_SLEW_DDR_PDENA, ENABLE);

	// config ddr timing.
	// SDCR = 0x0893C622 = 0000.1.0.0.0.1.0.0.1.0.0.1.1.1.1.00.011.0.0.010.0.010
	ddr2Regs->SDCR = 0x0893C622;
	ddr2Regs->SDCR &= ((ddr2Regs->SDCR & 0xFF0FFFFF) | 0x00800000);
	ddr2Regs->SDCR = ((ddr2Regs->SDCR & 0xFF0FFFFF) | 0x02000000);

	ddr2Regs->SDCR2 = 0x0;

	ddr2Regs->SDTIMR1 = 0x20923A89;
	ddr2Regs->SDTIMR2 = 0x0015C720;

	ddr2Regs->SDCR &= (~0x00008000);

	ddr2Regs->SDRCR = 0x00000000;
	CSL_FINS(ddr2Regs->SDRCR, DDR2_MDDR_SDRCR_RR, 0x00000492);

	// set ddr2 to sync reset.
	CSL_FINST(ddr2Regs->SDRCR, DDR2_MDDR_SDRCR_LPMODEN, NO_LPMODE);
	CSL_FINST(ddr2Regs->SDRCR, DDR2_MDDR_SDRCR_MCLKSTOPEN, MCLKSTOP_EN);

	// sync reset the ddr clock.
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_DDR2_MDDR, CSL_PSC_MDCTL_NEXT_SYNCRST);

	// enable the clock.
	PSC_MudaEstadoPerif(psc1Regs, CSL_PSC_PTCMD_GO0_MASK, CSL_PSC_DDR2_MDDR, CSL_PSC_MDCTL_NEXT_ENABLE);

	// disable self refresh.
	CSL_FINST(ddr2Regs->SDRCR, DDR2_MDDR_SDRCR_LPMODEN, LPMODE);
	CSL_FINST(ddr2Regs->SDRCR, DDR2_MDDR_SDRCR_MCLKSTOPEN, MCLKSTOP_DIS);

	// bloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x00000000;
	syscfgRegs->KICK1R = 0x00000000;
}

/**
 * \fn void EMIFA_Inicializa()
 * \brief Funo que inicializa o EMIFA e configura para acesso aos perifricos mapeados em memria.
 *
 * Esta funo configura os espaos de memria mapeados da EMIFA (External Memory Interface A) para serem usados
 * no acesso aos perifricos mapeados em memria do sistema.
 * Nesta funo  realizada a configurao dos pinos compartilhados para que possam ser usados pela EMIFA e tambm
 *  definida a fonte do clock de referncia da EMIFA (PLL0_SYSCLK3).
 * Os espaos correspondentes ao CS2 e CS3 so configurados para acesso a memria assncrona
 * 		CS2 -> Placa de Aquisio de Ultrassom v2.0 (0x60000000 - 0x61FFFFFF)
 * 		CS3 -> Placa de Interface USB (0x62000000 - 0x63FFFFFF)
 *
 */
void EMIFA_Inicializa()
{
	// desbloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x83E70B13;
	syscfgRegs->KICK1R = 0x95A4F1E0;

	// configura os pinos para serem usados pela EMIFA
	syscfgRegs->PINMUX5 &= 0x00FFFFFF;
	syscfgRegs->PINMUX5 |= 0x11000000;
	syscfgRegs->PINMUX6 = 0x11111111;
	syscfgRegs->PINMUX7 = 0x11111111;
	syscfgRegs->PINMUX8 = 0x11111111;
	syscfgRegs->PINMUX9 = 0x11111111;
	syscfgRegs->PINMUX10 = 0x11111111;
	syscfgRegs->PINMUX11 = 0x11111111;
	syscfgRegs->PINMUX12 = 0x11111111;

	// configura a fonte do clock base da EMIFA (PLL0_SYSCLK3)
	CSL_FINST(syscfgRegs->CFGCHIP3, SYSCFG_CFGCHIP3_EMA_CLKSRC, PLLCTRL_SYSCLK3);

	// cria as configuraes do espao de memria correspondente ao CS2
	emifCe2cfgSpeedDown = CSL_FMKT(EMIFA_CE2CFG_SS, SELSTRB_DISABLE) | CSL_FMKT(EMIFA_CE2CFG_EW, EXT_WAIT_DISABLE)
			| CSL_FMK(EMIFA_CE2CFG_W_SETUP, 2) | CSL_FMK(EMIFA_CE2CFG_W_STROBE, 8) | CSL_FMK(EMIFA_CE2CFG_W_HOLD, 2)
			| CSL_FMK(EMIFA_CE2CFG_R_SETUP, 0) | CSL_FMK(EMIFA_CE2CFG_R_STROBE, 4) | CSL_FMK(EMIFA_CE2CFG_R_HOLD, 0)
			| CSL_FMK(EMIFA_CE2CFG_TA, 0) | CSL_FMKT(EMIFA_CE2CFG_ASIZE, 16BIT); // configurao para acesso lento
	emifCe2cfgSpeedUp = CSL_FMKT(EMIFA_CE2CFG_SS, SELSTRB_DISABLE) | CSL_FMKT(EMIFA_CE2CFG_EW, EXT_WAIT_DISABLE)
			| CSL_FMK(EMIFA_CE2CFG_W_SETUP, 2) | CSL_FMK(EMIFA_CE2CFG_W_STROBE, 4) | CSL_FMK(EMIFA_CE2CFG_W_HOLD, 2)
			| CSL_FMK(EMIFA_CE2CFG_R_SETUP, 0) | CSL_FMK(EMIFA_CE2CFG_R_STROBE, 4) | CSL_FMK(EMIFA_CE2CFG_R_HOLD, 0)
			| CSL_FMK(EMIFA_CE2CFG_TA, 0) | CSL_FMKT(EMIFA_CE2CFG_ASIZE, 16BIT); // configurao para acesso rpido

	// configura o espao de memria correspondente ao CS2
	emifaRegs->CE2CFG = emifCe2cfgSpeedDown;

	// configura o espao de memria correspondente ao CS3
	emifaRegs->CE3CFG = CSL_FMKT(EMIFA_CE3CFG_SS, SELSTRB_DISABLE) | CSL_FMKT(EMIFA_CE3CFG_EW, EXT_WAIT_DISABLE)
			| CSL_FMK(EMIFA_CE3CFG_W_SETUP, 2) | CSL_FMK(EMIFA_CE3CFG_W_STROBE, 6) | CSL_FMK(EMIFA_CE3CFG_W_HOLD, 3)
			| CSL_FMK(EMIFA_CE3CFG_R_SETUP, 3) | CSL_FMK(EMIFA_CE3CFG_R_STROBE, 9) | CSL_FMK(EMIFA_CE3CFG_R_HOLD, 2)
			| CSL_FMK(EMIFA_CE3CFG_TA, 3) | CSL_FMKT(EMIFA_CE3CFG_ASIZE, 16BIT);

	// bloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x00000000;
	syscfgRegs->KICK1R = 0x00000000;
}

/**
 * \fn void EMIFA_Cs2DiminuiVeloc()
 * \brief Funo que reconfigura o espao de memria correspondente ao CS2 para usar a configurao de
 * velocidade mais baixa.
 *
 */
void EMIFA_Cs2DiminuiVeloc()
{
	Uint32 i;

	// reconfigura o espao de memria correspondente ao CS2 para uma velocidade de acesso mais baixa
	emifaRegs->CE2CFG = emifCe2cfgSpeedDown;

	// aguarda um tempo para a configurao estabilizar
	for (i = 0; i < 10000; i++);
}

/**
 * \fn void EMIFA_Cs2AumentaVeloc()
 * \brief Funo que reconfigura o espao de memria correspondente ao CS2 para usar a configurao de
 * velocidade mais alta.
 *
 */
void EMIFA_Cs2AumentaVeloc()
{
	Uint32 i;

	// reconfigura o espao de memria correspondente ao CS2 para uma velocidade de acesso mais alta
	emifaRegs->CE2CFG = emifCe2cfgSpeedUp;

	// aguarda um tempo para a configurao estabilizar
	for (i = 0; i < 10000; i++);
}

/**
 * \fn void DSPINTC_Inicializa()
 * \brief Funo que inicializa o controlador de interrupes do DSP.
 *
 * Esta funo configura o INTC (C674x Megamodule Interrupt Controller) para tratar as interrupes do sistema.
 * Na funo, a localizao do incio do vetor de tratamento de interrupes  fornecido ao controlador.
 * Tambm so configurados quais eventos iro disparar as interrupes de cada vetor, e os vetores ativos so habilitados.
 * As interrupes configuradas so:
 * 		- Interrupo de Excitao (Setada pela Controladora de Excitao para indicar que os transdutores foram excitados).
 * 		- Interrupo de EDMA (Setada pela EDMA para indicar que uma transferncia foi completada).
 *
 */
void DSPINTC_Inicializa()
{
	// desabilita a interrupo global
	CSR &= ~(0x1);

	// mapeia o evento de interrupo da Excitao para o vetor de interrupo 4
	CSL_FINS(intcRegs->INTMUX1, DSPINTC_INTMUX1_INTSEL4, GPIO_EXCIT_INT_EVT);

	// mapeia o evento de interrupo da EDMA responsvel pela transferncia de dados das FIFO's dos AD's para o vetor de interrupo 12
	CSL_FINS(intcRegs->INTMUX1, DSPINTC_INTMUX3_INTSEL12, QDMA_FIFO_AD_EVT);

	// indica para o controlador a posio do vetor de tratamento de interrupes
	ISTP = (unsigned int)_intcVectorTable;

	// limpa eventuais eventos de interrupo disparados anteriormente
	ICR = 0xFFF0;

	// habilita as interrupes bsicas do sistema
	IER = 0x0002;
	// habilita o vetor de interrupo 4
	IER |= (0x1 << 4);
	// habilita o vetor de interrupo 12
	IER |= (0x1 << 12);

	// habilita a interrupo global
	CSR |= (0x1);
}

/**
 * \fn void DSPINTC_LimpaEvento(Uint8 vetor, Uint8 evento)
 * \brief Funo que limpa um evento de interrupo.
 *
 * Limpa o flag de um evento de interrupo gerado e tambm o flag do vetor de tratamento para o qual
 * o evento est mapeado.
 *
 * \param Uint8 vetor nmero do vetor de tratamento para o qual o evento est mapeado.
 * \param Uint8 evento nmero do evento a ter o flag limpo.
 *
 */
void DSPINTC_LimpaEvento(Uint8 vetor, Uint8 evento)
{
	Uint8 evtReg; ///< varivel para indicar em qual dos 4 registradores possveis o evento est situado

	// verifica em qual dos 4 registradores possveis o evento est situado e converte o evento para representar o respectivo bit dentro do registrador
	if (evento < 32)
	{
		evtReg = 0;
	}
	else if (evento < 64)
	{
		evtReg = 1;
		evento -= 32;
	}
	else if (evento < 96)
	{
		evtReg = 2;
		evento -= 64;
	}
	else if (evento < 128)
	{
		evtReg = 3;
		evento -= 96;
	}

	// limpa o flag do vetor para o qual o evento est mapeado
	ICR = (0x1 << vetor);

	// limpa o flag do evento
	intcRegs->EVTCLR[evtReg] = (0x1 << evento);
}

/**
 * \fn void EDMA3_Inicializa()
 * \brief Funo que inicializa o EDMA3 (Enhanced Direct Memory Access Controller).
 *
 * Esta funo configura o EDMA3 (Enhanced Direct Memory Access Controller) definindo os canais que sero utilizados
 * nas transferncias.
 * Como a interrupo da EDMA3 que est mapeada para o controlador de interrupes do DSP  a interrupo do Shadow
 * Region 1, os canais usados pelo sistema sero mapeados para esta Shadow Region.
 *
 * Dentre as transferncias temos:
 * 		- Cpia dos dados das FIFO's dos AD's para memria externa DDR2;
 *
 */
void EDMA3_Inicializa()
{
	// mapeia o canal da QDMA usado nas transferncias das FIFO's para a Shadow Region 1.
	edma30ccRegs->QRAE[1] |= (0x1 << QDMA_FIFO_AD_CHANNEL);

	// configura o canal da QDMA escolhido para uso nas transferncias de dados das FIFO's dos AD's para a memria externa DDR2.
	CSL_FINS(edma30ccRegs->QCHMAP[QDMA_FIFO_AD_CHANNEL], EDMA3CC_QCHMAP_PAENTRY, QDMA_FIFO_AD_PARAM_ENTRY);	// configura o canal da QDMA para usar o conjunto de parmetros 32.
	CSL_FINS(edma30ccRegs->QCHMAP[QDMA_FIFO_AD_CHANNEL], EDMA3CC_QCHMAP_TRWORD, 7);	// configura o canal da QDMA para ser disparada na escrita do ltimo parmetro.
	CSL_FINST(edma30ccRegs->QDMAQNUM, EDMA3CC_QDMAQNUM_E0, Q0);	// configura o canal da QDMA para ser enfileirada na fila Q0.
	edma30ccRegs->SHADOW[1].QEESR |= (0x1 << QDMA_FIFO_AD_CHANNEL);	// habilita o canal da QDMA para gerar eventos.

	// habilita a gerao de interrupo de trmino da transferncia.
	edma30ccRegs->SHADOW[1].IESR = (0x1 << QDMA_FIFO_AD_TCC);

	// limpa eventuais interrupes previamente disparadas.
	edma30ccRegs->SHADOW[1].ICR = 0xFFFFFFFF;
}

/**
 * \fn Uint8 EDMA3_LeInt()
 * \brief Funo que l o registrador de interrupes pendentes da EDMA.
 *
 * Esta funo l o valor presente no registrador de interrupes pendentes IPR da EDMA para que
 * se possa determinar qual o canal que gerou a interrupo, a partir do cdigo de transferncia
 * completada TCC.
 *
 * \return valor do IPR com o flag que gerou a interrupo.
 *
 */
Uint32 EDMA3_LeInt()
{
	return edma30ccRegs->SHADOW[1].IPR;
}

/**
 * \fn void EDMA3_LimpaInt(Uint8 tcc)
 * \brief Funo que limpa o flag de uma interrupo da EDMA.
 *
 * Esta funo limpa o flag da interrupo da EDMA setado no registrador de interrupes pendentes
 * que foi gerada de acordo com o cdigo de transferncia completada TCC fornecido.
 *
 * \param Uint8 tcc cdigo de transferncia completada da interrupo cujo flag deve ser limpo.
 *
 */
void EDMA3_LimpaInt(Uint8 tcc)
{
	edma30ccRegs->SHADOW[1].ICR = (0x1 << tcc);
}

/**
 * \fn void EDMA3_ConfFifoAd(Uint32 tamanhoAmostras, Uint16 *destinoCanal1, Uint16 *destinoCanal2, Uint16 *destinoCanal3)
 * \brief Funo que configura a transferncia via EDMA das FIFO's dos AD's para a memria externa DDR2.
 *
 * Esta funo inicializa a Parameter RAM (PaRAM) com os dados das transferncias de dados das FIFO's dos AD's
 * para a memria externa DDR2.
 * Os canais que no devem ser considerados na aquisio, devero ter seus repectivos endereos de destino setados
 * como NULL.
 *
 * \param Uint32 tamanhoAmostras quantidade de amostras que sero transferidas da FIFO
 * \param Uint16 *destinoCanal1 endereo do buffer de destino das amostras provenientes do Canal 1 da Aquisio
 * \param Uint16 *destinoCanal2 endereo do buffer de destino das amostras provenientes do Canal 2 da Aquisio
 * \param Uint16 *destinoCanal3 endereo do buffer de destino das amostras provenientes do Canal 3 da Aquisio
 *
 */
void EDMA3_ConfFifoAd(Uint32 tamanhoAmostras, Uint16 *destinoCanal1, Uint16 *destinoCanal2, Uint16 *destinoCanal3)
{
	Uint8 proximoParamEntry = 0;	///< indicador de qual o prximo PaRAM Entry no qual deve ser escrito o PaRAM Set
	volatile CSL_Edma3ccParamSetRegs *paramSetRegs = &paramInicialCopiaFifos;	///< conjunto de registradores do PaRAM Set sendo configurado

	// verifica se o Canal 1 possui destino e deve ter seus dados adquiridos
	if (destinoCanal1 != NULL)
	{
		// verifica se o primeiro PaRAM Entry j foi configurado
		if (proximoParamEntry)
		{
			// aponta para os registradores do prximo PaRAM Entry
			paramSetRegs = &edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY + proximoParamEntry];
		}

		// monta o PaRAM Set com as configuraes da transferncia
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_ITCCHEN, DISABLE);	// desabilita o encadeamento a partir de Intermediate Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCCHEN, DISABLE);	// desabilita o encadeamento a partir de Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_ITCINTEN, DISABLE);	// desabilita a gerao de interrupo de Intermediate Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCINTEN, DISABLE);	// desabilita a gerao de interrupo de Transfer Complete
		CSL_FINS(paramSetRegs->OPT, EDMA3CC_OPT_TCC, QDMA_FIFO_AD_TCC);	// define o cdigo que identifica qual transferncia foi completada
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCCMOD, NORMAL);	// define o modo usado para considerar a transferncia como completada para Normal
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_FWID, 16BIT);	// define a largura da FIFO como 16 bits (a fonte dos dados  uma FIFO)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_STATIC, NORMAL);	// seta a transferncia como no esttica, ou seja, faz o Link com uma prxima
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_SYNCDIM, ABSYNC);	// define o modo de de sincronia da transferncia como AB (transfere BCNT arrays de ACNT bytes)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_DAM, INCR);	// define que o endereo de destino deve ser incrementado (no  uma FIFO)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_SAM, CONST);	// define que o endereo de fonte no deve ser incrementado ( uma FIFO)

		paramSetRegs->SRC = CS_AD_1;	// define o endereo da fonte dos dados como a FIFO do AD1

		// configura a quantidade de arrays a tranferir e o tamanho dos arrays. Isso foi feito conforme recomentao do manual da EDMA3 (seo 2.3.7.2 Element Size, pgina 29)
		CSL_FINS(paramSetRegs->A_B_CNT, EDMA3CC_A_B_CNT_BCNT, tamanhoAmostras);	// seta a quantidade de arrays que sero transferidos conforme tamanhoAmostras
		CSL_FINS(paramSetRegs->A_B_CNT, EDMA3CC_A_B_CNT_ACNT, sizeof(*destinoCanal1));	// seta o tamanho dos arrays como o tamanho dos dados no buffer de destino

		paramSetRegs->DST = (Uint32)destinoCanal1;	// define o endereo do destino dos dados como o endereo passado em destinoCanal1

		CSL_FINS(paramSetRegs->SRC_DST_BIDX, EDMA3CC_SRC_DST_BIDX_DSTBIDX, sizeof(*destinoCanal1));	// define de quanto se deve incrementar o endereo do destino entre um array e outro
		CSL_FINS(paramSetRegs->SRC_DST_BIDX, EDMA3CC_SRC_DST_BIDX_SRCBIDX, 0);	// define de quanto se deve incrementar o endereo da fonte entre um array e outro (0 pois  uma FIFO)

		CSL_FINS(paramSetRegs->LINK_BCNTRLD, EDMA3CC_LINK_BCNTRLD_BCNTRLD, tamanhoAmostras);	// define o valor a recarregar em BCNT ao trmino do frame (nmero de frames definido por CCNT)
		CSL_FINS(paramSetRegs->LINK_BCNTRLD, EDMA3CC_LINK_BCNTRLD_LINK, (Uint32)&edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY + proximoParamEntry]);	// define o PaRAM Entry que contm as configuraes da prxima tranferncia linkada

		CSL_FINS(paramSetRegs->SRC_DST_CIDX, EDMA3CC_SRC_DST_CIDX_DSTCIDX, 0);	// define de quanto se deve incrementar o endereo do destino entre um frame e outro (0 pois  apenas 1 frame)
		CSL_FINS(paramSetRegs->SRC_DST_CIDX, EDMA3CC_SRC_DST_CIDX_SRCCIDX, 0);	// define de quanto se deve incrementar o endereo da fonte entre um frame e outro (0 pois  apenas 1 frame)

		CSL_FINS(paramSetRegs->CCNT, EDMA3CC_CCNT_CCNT, 1);	// seta a quantidade de frames que sero transferidos (apenas 1)

		// incrementa o indicador do prximo PaRAM Entry
		proximoParamEntry++;
	}

	// verifica se o Canal 2 possui destino e deve ter seus dados adquiridos
	if (destinoCanal2 != NULL)
	{
		// verifica se o primeiro PaRAM Entry j foi configurado
		if (proximoParamEntry)
		{
			// aponta para os registradores do prximo PaRAM Entry
			paramSetRegs = &edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY + proximoParamEntry];
		}

		// monta o PaRAM Set com as configuraes da transferncia
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_ITCCHEN, DISABLE);	// desabilita o encadeamento a partir de Intermediate Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCCHEN, DISABLE);	// desabilita o encadeamento a partir de Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_ITCINTEN, DISABLE);	// desabilita a gerao de interrupo de Intermediate Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCINTEN, DISABLE);	// desabilita a gerao de interrupo de Transfer Complete
		CSL_FINS(paramSetRegs->OPT, EDMA3CC_OPT_TCC, QDMA_FIFO_AD_TCC);	// define o cdigo que identifica qual transferncia foi completada
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCCMOD, NORMAL);	// define o modo usado para considerar a transferncia como completada para Normal
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_FWID, 16BIT);	// define a largura da FIFO como 16 bits (a fonte dos dados  uma FIFO)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_STATIC, NORMAL);	// seta a transferncia como no esttica, ou seja, faz o Link com uma prxima
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_SYNCDIM, ABSYNC);	// define o modo de de sincronia da transferncia como AB (transfere BCNT arrays de ACNT bytes)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_DAM, INCR);	// define que o endereo de destino deve ser incrementado (no  uma FIFO)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_SAM, CONST);	// define que o endereo de fonte no deve ser incrementado ( uma FIFO)

		paramSetRegs->SRC = CS_AD_2;	// define o endereo da fonte dos dados como a FIFO do AD2

		// configura a quantidade de arrays a tranferir e o tamanho dos arrays. Isso foi feito conforme recomentao do manual da EDMA3 (seo 2.3.7.2 Element Size, pgina 29)
		CSL_FINS(paramSetRegs->A_B_CNT, EDMA3CC_A_B_CNT_BCNT, tamanhoAmostras);	// seta a quantidade de arrays que sero transferidos conforme tamanhoAmostras
		CSL_FINS(paramSetRegs->A_B_CNT, EDMA3CC_A_B_CNT_ACNT, sizeof(*destinoCanal2));	// seta o tamanho dos arrays como o tamanho dos dados no buffer de destino

		paramSetRegs->DST = (Uint32)destinoCanal2;	// define o endereo do destino dos dados como o endereo passado em destinoCanal2

		CSL_FINS(paramSetRegs->SRC_DST_BIDX, EDMA3CC_SRC_DST_BIDX_DSTBIDX, sizeof(*destinoCanal2));	// define de quanto se deve incrementar o endereo do destino entre um array e outro
		CSL_FINS(paramSetRegs->SRC_DST_BIDX, EDMA3CC_SRC_DST_BIDX_SRCBIDX, 0);	// define de quanto se deve incrementar o endereo da fonte entre um array e outro (0 pois  uma FIFO)

		CSL_FINS(paramSetRegs->LINK_BCNTRLD, EDMA3CC_LINK_BCNTRLD_BCNTRLD, tamanhoAmostras);	// define o valor a recarregar em BCNT ao trmino do frame (nmero de frames definido por CCNT)
		CSL_FINS(paramSetRegs->LINK_BCNTRLD, EDMA3CC_LINK_BCNTRLD_LINK, (Uint32)&edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY + proximoParamEntry]);	// define o PaRAM Entry que contm as configuraes da prxima tranferncia linkada

		CSL_FINS(paramSetRegs->SRC_DST_CIDX, EDMA3CC_SRC_DST_CIDX_DSTCIDX, 0);	// define de quanto se deve incrementar o endereo do destino entre um frame e outro (0 pois  apenas 1 frame)
		CSL_FINS(paramSetRegs->SRC_DST_CIDX, EDMA3CC_SRC_DST_CIDX_SRCCIDX, 0);	// define de quanto se deve incrementar o endereo da fonte entre um frame e outro (0 pois  apenas 1 frame)

		CSL_FINS(paramSetRegs->CCNT, EDMA3CC_CCNT_CCNT, 1);	// seta a quantidade de frames que sero transferidos (apenas 1)

		// incrementa o indicador do prximo PaRAM Entry
		proximoParamEntry++;
	}

	// verifica se o Canal 3 possui destino e deve ter seus dados adquiridos
	if (destinoCanal3 != NULL)
	{
		// verifica se o primeiro PaRAM Entry j foi configurado
		if (proximoParamEntry)
		{
			// aponta para os registradores do prximo PaRAM Entry
			paramSetRegs = &edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY + proximoParamEntry];
		}

		// monta o PaRAM Set com as configuraes da transferncia
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_ITCCHEN, DISABLE);	// desabilita o encadeamento a partir de Intermediate Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCCHEN, DISABLE);	// desabilita o encadeamento a partir de Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_ITCINTEN, DISABLE);	// desabilita a gerao de interrupo de Intermediate Transfer Complete
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCINTEN, DISABLE);	// desabilita a gerao de interrupo de Transfer Complete
		CSL_FINS(paramSetRegs->OPT, EDMA3CC_OPT_TCC, QDMA_FIFO_AD_TCC);	// define o cdigo que identifica qual transferncia foi completada
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCCMOD, NORMAL);	// define o modo usado para considerar a transferncia como completada para Normal
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_FWID, 16BIT);	// define a largura da FIFO como 16 bits (a fonte dos dados  uma FIFO)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_STATIC, NORMAL);	// seta a transferncia como no esttica, ou seja, faz o Link com uma prxima
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_SYNCDIM, ABSYNC);	// define o modo de de sincronia da transferncia como AB (transfere BCNT arrays de ACNT bytes)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_DAM, INCR);	// define que o endereo de destino deve ser incrementado (no  uma FIFO)
		CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_SAM, CONST);	// define que o endereo de fonte no deve ser incrementado ( uma FIFO)

		paramSetRegs->SRC = CS_AD_3;	// define o endereo da fonte dos dados como a FIFO do AD3

		// configura a quantidade de arrays a tranferir e o tamanho dos arrays. Isso foi feito conforme recomentao do manual da EDMA3 (seo 2.3.7.2 Element Size, pgina 29)
		CSL_FINS(paramSetRegs->A_B_CNT, EDMA3CC_A_B_CNT_BCNT, tamanhoAmostras);	// seta a quantidade de arrays que sero transferidos conforme tamanhoAmostras
		CSL_FINS(paramSetRegs->A_B_CNT, EDMA3CC_A_B_CNT_ACNT, sizeof(*destinoCanal3));	// seta o tamanho dos arrays como o tamanho dos dados no buffer de destino

		paramSetRegs->DST = (Uint32)destinoCanal3;	// define o endereo do destino dos dados como o endereo passado em destinoCanal3

		CSL_FINS(paramSetRegs->SRC_DST_BIDX, EDMA3CC_SRC_DST_BIDX_DSTBIDX, sizeof(*destinoCanal3));	// define de quanto se deve incrementar o endereo do destino entre um array e outro
		CSL_FINS(paramSetRegs->SRC_DST_BIDX, EDMA3CC_SRC_DST_BIDX_SRCBIDX, 0);	// define de quanto se deve incrementar o endereo da fonte entre um array e outro (0 pois  uma FIFO)

		CSL_FINS(paramSetRegs->LINK_BCNTRLD, EDMA3CC_LINK_BCNTRLD_BCNTRLD, tamanhoAmostras);	// define o valor a recarregar em BCNT ao trmino do frame (nmero de frames definido por CCNT)
		CSL_FINS(paramSetRegs->LINK_BCNTRLD, EDMA3CC_LINK_BCNTRLD_LINK, (Uint32)&edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY + proximoParamEntry]);	// define o PaRAM Entry que contm as configuraes da prxima tranferncia linkada

		CSL_FINS(paramSetRegs->SRC_DST_CIDX, EDMA3CC_SRC_DST_CIDX_DSTCIDX, 0);	// define de quanto se deve incrementar o endereo do destino entre um frame e outro (0 pois  apenas 1 frame)
		CSL_FINS(paramSetRegs->SRC_DST_CIDX, EDMA3CC_SRC_DST_CIDX_SRCCIDX, 0);	// define de quanto se deve incrementar o endereo da fonte entre um frame e outro (0 pois  apenas 1 frame)

		CSL_FINS(paramSetRegs->CCNT, EDMA3CC_CCNT_CCNT, 1);	// seta a quantidade de frames que sero transferidos (apenas 1)

		// incrementa o indicador do prximo PaRAM Entry
		proximoParamEntry++;
	}

	// faz as configuraes especficas da ltima transferncia do Link
	CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_TCINTEN, ENABLE);	// habilita a gerao de interrupo de Transfer Complete
	CSL_FINST(paramSetRegs->OPT, EDMA3CC_OPT_STATIC, STATIC);	// seta a transferncia como esttica, ou seja, no faz o Link com uma prxima
	CSL_FINS(paramSetRegs->LINK_BCNTRLD, EDMA3CC_LINK_BCNTRLD_LINK, 0xFFFF);	// define o PaRAM Entry que contm as configuraes da prxima tranferncia linkada como NULO
}

/**
 * \fn void EDMA3_DisparaFifoAd()
 * \brief Funo que dispara a transferncia de dados das FIFO's dos AD's atravs da QDMA.
 *
 * Esta funo fora o incio da transferncia de dados das FIFO's dos AD's para a memria externa DDR2
 * atravs do canal da QDMA configurado para tal funo.
 * Para dar incio  transferncia,  configurado o PaRAM Entry mapeado para o canal da QDMA usado,
 * cuja escrita dispara o canal.
 *
 */
void EDMA3_DisparaFifoAd()
{
	// atualiza as configuraes do PaRAM Entry do canal QDMA responsvel pela transferncia
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].OPT = paramInicialCopiaFifos.OPT;
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].SRC = paramInicialCopiaFifos.SRC;
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].A_B_CNT = paramInicialCopiaFifos.A_B_CNT;
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].DST = paramInicialCopiaFifos.DST;
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].SRC_DST_BIDX = paramInicialCopiaFifos.SRC_DST_BIDX;
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].LINK_BCNTRLD = paramInicialCopiaFifos.LINK_BCNTRLD;
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].SRC_DST_CIDX = paramInicialCopiaFifos.SRC_DST_CIDX;

	// realiza a escrita do ltimo parmetro do PaRAM Entry, disparando a transferncia
	edma30ccRegs->PARAMSET[QDMA_FIFO_AD_PARAM_ENTRY].CCNT = paramInicialCopiaFifos.CCNT;
}

/**
 * \fn void SPI_Inicializa()
 * \brief Funo que inicializa as SPI's (Serial Peripheral Interface) conforme as necessidades.
 *
 * Esta funo configura as SPI's (Serial Peripheral Interface) para sere usadas na comunicao com os perifricos
 * da Placa de Aquisio v2.0, como os conversores A/D dos canais de aquisio e o conversor A/D da entrada do PT100.
 * Nesta funo  realizada a configurao dos pinos compartilhados para que possam ser usados pela SPI.
 * Atualmente, apenas a SPI1 est sendo utilizada na comunicao com o sistema.
 *
 */
void SPI_Inicializa()
{
	// desbloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x83E70B13;
	syscfgRegs->KICK1R = 0x95A4F1E0;

	// configura os pinos para serem usados pela SPI1
	syscfgRegs->PINMUX4 &= 0x00FFFFFF;
	syscfgRegs->PINMUX4 |= 0x11000000;
	syscfgRegs->PINMUX5 &= 0xFF0000FF;
	syscfgRegs->PINMUX5 |= 0x00111100;

	// primeiramente, reseta o perifrico SPI1
	CSL_FINST(spi1Regs->SPIGCR0, SPI_SPIGCR0_RESET, IN_RESET);

	// retira a SPI1 do reset
	CSL_FINST(spi1Regs->SPIGCR0, SPI_SPIGCR0_RESET, OUT_OF_RESET);

	// coloca a SPI1 em modo Master (CLKMOD = 1 e MASTER = 1)
	CSL_FINST(spi1Regs->SPIGCR1, SPI_SPIGCR1_MASTER, MASTER);
	CSL_FINST(spi1Regs->SPIGCR1, SPI_SPIGCR1_CLKMOD, INTERNAL);

	// configura os pinos CLK, SOMI, SIMO, SCS2 e SCS3 para serem usados como pinos de SPI
	CSL_FINST(spi1Regs->SPIPC0, SPI_SPIPC0_SOMIFUN, SPI);
	CSL_FINST(spi1Regs->SPIPC0, SPI_SPIPC0_SIMOFUN, SPI);
	CSL_FINST(spi1Regs->SPIPC0, SPI_SPIPC0_CLKFUN, SPI);
	CSL_FINST(spi1Regs->SPIPC0, SPI_SPIPC0_SCS0FUN2, SPI);
	CSL_FINST(spi1Regs->SPIPC0, SPI_SPIPC0_SCS0FUN3, SPI);

	// configura o estado default dos chip selects em uso para ficar em alto
	spi1Regs->SPIDEF |= (0x1 << SPI_AQUIS_SCSN) | (0x1 << SPI_EXCIT_SCSN);

	// configura o formato dos dados para comunicao com os AD's da aquisio
	CSL_FINS(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_WDELAY, 3);
	CSL_FINST(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_PARENA, DISABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_WAITENA, DISABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_SHIFTDIR, MSB);
	CSL_FINST(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_DISCSTIMERS, ENABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_POLARITY, HIGH);
	CSL_FINST(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_PHASE, DELAY);
	CSL_FINS(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_PRESCALE, 149);	// frequencia dada por PLL0_SYSCLK2 / PRESCALE
	CSL_FINS(spi1Regs->SPIFMT[SPI_AQUIS_AD_FORMAT], SPI_SPIFMT_CHARLEN, 16);

	// configura o formato dos dados para comunicao com o AD do PT100
	CSL_FINS(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_WDELAY, 3);
	CSL_FINST(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_PARENA, DISABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_WAITENA, DISABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_SHIFTDIR, MSB);
	CSL_FINST(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_DISCSTIMERS, ENABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_POLARITY, HIGH);
	CSL_FINST(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_PHASE, DELAY);
	CSL_FINS(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_PRESCALE, 149);	// frequencia dada por PLL0_SYSCLK2 / PRESCALE
	CSL_FINS(spi1Regs->SPIFMT[SPI_PT100_AD_FORMAT], SPI_SPIFMT_CHARLEN, 16);

	// configura o formato dos dados para comunicao com o Controlador de Excitao
	CSL_FINS(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_WDELAY, 10);
	CSL_FINST(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_PARENA, DISABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_WAITENA, DISABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_SHIFTDIR, MSB);
	CSL_FINST(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_DISCSTIMERS, ENABLE);
	CSL_FINST(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_POLARITY, HIGH);
	CSL_FINST(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_PHASE, DELAY);
	CSL_FINS(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_PRESCALE, 149);	// frequencia dada por PLL0_SYSCLK2 / PRESCALE
	CSL_FINS(spi1Regs->SPIFMT[SPI_EXCIT_FORMAT], SPI_SPIFMT_CHARLEN, 16);

	// desabilita as interrupes
	spi1Regs->SPIINT0 = 0x00000000;

	// habilita o perifrico SPI1 para uso
	CSL_FINST(spi1Regs->SPIGCR1, SPI_SPIGCR1_ENABLE, ENABLE);

	// desbloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x00000000;
	syscfgRegs->KICK1R = 0x00000000;
}

/**
 * \fn void SPI_Envia(Uint8 spiNum, Uint8 regFormato, Uint8 scsNum, Uint16 dado)
 * \brief Funo que envia um dado atravs da SPI.
 *
 * Esta funo envia um dado pela SPI, conforme as configuraes de formato passadas.
 *
 * \param Uint8 spiNum nmero da interface SPI a ser usada.
 * \param Uint8 regFormato nmero do registrador configurado com o formato de dados a ser utilizado no envio.
 * \param Uint8 scsNum nmero do pino de chip select da SPI que deve ser usado na transferncia.
 * \param Uint16 dado dado a ser enviado pela SPI.
 *
 */
void SPI_Envia(Uint8 spiNum, Uint8 regFormato, Uint8 scsNum, Uint16 dado)
{
	CSL_SpiRegsOvly spiRegs;
	// verifica qual a interface SPI ser usada na transferncia a aponta para o conjunto de registradores correspondentes
	if (spiNum == CSL_SPI_0)
	{
		// SPI0 no est sendo usada no momento
		return;
	}
	else if (spiNum == CSL_SPI_1)
	{
		spiRegs = spi1Regs;
	}
	else
	{
		// erro, nmero da SPI invlido
		return;
	}

	// escreve no registrador de dados da SPI as configuraes do envio e o dado a ser enviado
	spiRegs->SPIDAT1 = CSL_FMKT(SPI_SPIDAT1_CSHOLD, DISABLE) | CSL_FMKT(SPI_SPIDAT1_WDEL, DISABLE)
		 | CSL_FMK(SPI_SPIDAT1_DFSEL, regFormato) | CSL_FMK(SPI_SPIDAT1_CSNR, 0x00)
		 | CSL_FMK(SPI_SPIDAT1_TXDATA, dado);
}

/**
 * \fn void SPI_VerificaEnvio()
 * \brief Funo que verifica o status de envio da SPI.
 *
 * Esta funo verifica o status de envio da SPI, indicando se  possvel enviar algo ou no.
 * Para fazer a verificao,  checado o status do buffer de transmisso da SPI, que se estiver
 * vazio indica que a SPI est disponvel para nova transmisso.
 *
 * \param Uint8 spiNum nmero da interface SPI a ser usada.
 *
 * \return 0 se a SPI estiver indisponvel para envio, 1 se estiver disponvel
 *
 */
Uint8 SPI_VerificaEnvio(Uint8 spiNum)
{
	CSL_SpiRegsOvly spiRegs;
	// verifica qual a interface SPI ser usada na transferncia a aponta para o conjunto de registradores correspondentes
	if (spiNum == CSL_SPI_0)
	{
		// SPI0 no est sendo usada no momento
		return 0;
	}
	else if (spiNum == CSL_SPI_1)
	{
		spiRegs = spi1Regs;
	}
	else
	{
		// erro, nmero da SPI invlido
		return 0;
	}

	// retorna o status do buffer de envio da SPI, 0 para buffer cheio e 1 para buffer vazio
	return CSL_FEXT(spiRegs->SPIFLG, SPI_SPIFLG_TXINTFLG);
}

/**
 * \fn void TIMER_Inicializa()
 * \brief Funo que inicializa os TIMERs que sero usados no sistema.
 *
 * Esta funo no est sendo usada no momento.
 *
 */
void TIMER_Inicializa()
{
	//timer2Regs;

	timer2Regs->TGCR = 0x00000000;
	timer2Regs->TCR = 0x00000000;

	//reset
	CSL_FINST(timer2Regs->TGCR, TMR_TGCR_TIM34RS, RESET); //poe em reset
	CSL_FINST(timer2Regs->INTCTLSTAT, TMR_INTCTLSTAT_EVTINTSTAT34, PEND); //limpa se interrupo aconteceu

	//1.
	CSL_FINST(timer2Regs->TGCR, TMR_TGCR_TIMMODE, 32BIT_UNCHAIN);

	//2.
	CSL_FINST(timer2Regs->TGCR, TMR_TGCR_TIM34RS, NO_RESET);

	//3.
	timer2Regs->TIM34 = 0;
	timer2Regs->PRD34 = 2400; // para dar um perodo de 10ms considerando o prescaler em 10

	//4.
	CSL_FINS(timer2Regs->TGCR, TMR_TGCR_PSC34, 9); //Prescaler setado em 10 (PSC34 = 9)

	//5.
	CSL_FINST(timer2Regs->TCR, TMR_TCR_ENAMODE34, EN_CONT); //Timer34 habilitado em modo contnuo

	//7. Interrupo
	CSL_FINST(timer2Regs->TGCR, TMR_TGCR_PLUSEN, ENABLE); //Faz com que as funcionalidades do timer sejam expandidas, para gerar interrupo
	CSL_FINST(timer2Regs->INTCTLSTAT, TMR_INTCTLSTAT_PRDINTEN34, ENABLE); //Habilita a interrupo.
}

/**
 * \fn void GPIO_Inicializa()
 * \brief Funo que inicializa os GPIO's (General Purpose Input/Output) que sero usados no sistema.
 *
 * Esta funo configura os pinos que sero utilizados como GPIO (General Purpose Input/Output).
 * Estes pinos so:
 * 		- Pino de entrada de interrupo proveniente do Controlador de Excitao (GP0_12 ou GP8_7 para INT5; GP0_13 ou GP1_12 para INT6; GP0_14 ou GP1_13 para INT7)
 * 		- Pino de entrada de interrupo proveniente da UART (GP0_9 ou GP1_9) (em princpio no  usado, ento ficar desconfigurado pois conflita ou com a UART2 ou com a EMAC MII)
 * 		- Pino de sada do Reset da CPLD (GP0_7, GP1_10 ou GP1_11)
 *
 */
void GPIO_Inicializa()
{
	// desbloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x83E70B13;
	syscfgRegs->KICK1R = 0x95A4F1E0;

	// configura os pinos para serem usados como GPIO (GP0_7, GP0_12, GP0_13, GP0_14)
	syscfgRegs->PINMUX0 &= 0xFFFF000F;
	syscfgRegs->PINMUX0 |= 0x00008880;
	syscfgRegs->PINMUX1 &= 0xFFFFFFF0;
	syscfgRegs->PINMUX1 |= 0x00000008;

	// configura o mdulo GPIO para o funcionamento desejado
	gpioRegs->BANK[GPIO_CPLD_MRS_BANK/2].DIR &= ~((1 << GPIO_CPLD_MRS_PIN) << (16*(GPIO_CPLD_MRS_BANK % 2))); // configura o GPIO desejado como sada para o Reset da CPLD
	gpioRegs->BANK[GPIO_EXCIT_INT_BANK/2].DIR &= ~((1 << GPIO_EXCIT_INT_PIN) << (16*(GPIO_EXCIT_INT_BANK % 2))); // configura o GP0_14 como entrada para interrupo (EXCIT_INT)

	// define o valor inicial da sada GPIO do Reset da CPLD como 1
	gpioRegs->BANK[GPIO_CPLD_MRS_BANK/2].SET_DATA |= ((1 << GPIO_CPLD_MRS_PIN) << (16*(GPIO_CPLD_MRS_BANK % 2)));

	// habilita a gerao de interrupes a partir do banco contendo a entrada de interrupo (EXCIT_INT)
	gpioRegs->BINTEN |= (1 << GPIO_EXCIT_INT_BANK);

	// configura o pino contendo a entrada de interrupo (EXCIT_INT) para gerar interrupo na borda de subida
	gpioRegs->BANK[GPIO_EXCIT_INT_BANK/2].SET_RIS_TRIG |= ((1 << GPIO_EXCIT_INT_PIN) << (16*(GPIO_EXCIT_INT_BANK % 2)));

	// limpa possveis eventos de interrupo gerados antecipadamente
	gpioRegs->BANK[GPIO_EXCIT_INT_BANK/2].INTSTAT = 0xFFFF << (16*(GPIO_CPLD_MRS_BANK % 2));

	// desbloqueia os registradores do System configuration Module para acesso
	syscfgRegs->KICK0R = 0x00000000;
	syscfgRegs->KICK1R = 0x00000000;
}

/**
 * \fn void GPIO_SetaPino(Uint8 bancoGpio, Uint8 pinoGpio, Uint8 valor)
 * \brief Funo que seta o valor de um pino da GPIO.
 *
 * Esta funo seta o valor de um determinado pino da GPIO conforme desejado.
 *
 * \param Uint8 bancoGpio banco da GPIO onde est situado o pino
 * \param Uint8 pinoGpio pino do banco da GPIO que ter seu valor setado
 * \param Uint8 valor valor a ser apresentado na sada do pino
 *
 */
void GPIO_SetaPino(Uint8 bancoGpio, Uint8 pinoGpio, Uint8 valor)
{
	// converte o banco de GPIO enviado para o ndice do conjunto de registradores BANK na estrutura de registradores da GPIO
	bancoGpio /= 2;

	// verifica se o pino  de sada
	if (gpioRegs->BANK[bancoGpio].DIR & ((1 << pinoGpio) << (16*(bancoGpio % 2))))
	{
		// pino  de entrada, no faz nada e retorna
		return;
	}
	else
	{
		// pino  de sada, muda o valor de acordo
		if (valor)
		{
			// seta a sada do pino para 1
			gpioRegs->BANK[bancoGpio].SET_DATA |= ((1 << pinoGpio) << (16*(bancoGpio % 2)));
		}
		else
		{
			// seta a sada do pino para 0
			gpioRegs->BANK[bancoGpio].CLR_DATA |= ((1 << pinoGpio) << (16*(bancoGpio % 2)));
		}
	}
}
